@@ -115,7 +115,7 @@ void smb_SetupIoctlFid(smb_fid_t *fidp, cm_space_t *prefix)
  * this is the first read call.  This is the function that actually makes the
  * call to the ioctl code.
  */
-smb_IoctlPrepareRead(smb_fid_t *fidp, smb_ioctl_t *ioctlp, cm_user_t *userp)
+long smb_IoctlPrepareRead(smb_fid_t *fidp, smb_ioctl_t *ioctlp, cm_user_t *userp)
 {
     long opcode;
     smb_ioctlProc_t *procp = NULL;
@@ -339,7 +339,7 @@ long smb_IoctlV3Read(smb_fid_t *fidp, smb_vc_t *vcp, smb_packet_t *inp, smb_pack
 	
     uidp = smb_FindUID(vcp, ((smb_t *)inp)->uid, 0);
     userp = smb_GetUserFromUID(uidp);
-    osi_assert(userp != 0);
+    osi_assertx(userp != NULL, "null cm_user_t");
     iop->uidp = uidp;
     if (uidp && uidp->unp) {
         osi_Log3(afsd_logp, "Ioctl uid %d user %x name %s",
