@@ -15,7 +15,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/UKERNEL/afs_usrops.c,v 1.27.2.2 2005/03/11 06:50:46 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/UKERNEL/afs_usrops.c,v 1.30.6.5 2007/10/31 22:31:57 shadow Exp $");
 
 
 #ifdef	UKERNEL
@@ -459,16 +459,12 @@ afs_osi_Sleep(void *x)
 {
     int index;
     osi_wait_t *waitp;
-    int rxGlockOwner = ISAFS_RXGLOCK();
     int glockOwner = ISAFS_GLOCK();
 
     usr_mutex_lock(&osi_waitq_lock);
     if (glockOwner) {
 	AFS_GUNLOCK();
     }
-    if (rxGlockOwner) {
-	AFS_RXGUNLOCK();
-    }
     index = WAITHASH(x);
     if (osi_waithash_avail == NULL) {
 	waitp = (osi_wait_t *) afs_osi_Alloc(sizeof(osi_wait_t));
@@ -495,9 +491,6 @@ afs_osi_Sleep(void *x)
     if (glockOwner) {
 	AFS_GLOCK();
     }
-    if (rxGlockOwner) {
-	AFS_RXGLOCK();
-    }
 }
 
 int
@@ -533,7 +526,6 @@ afs_osi_Wait(afs_int32 msec, struct afs_osi_WaitHandle *handle, int intok)
     osi_wait_t *waitp;
     struct timespec tv;
     int ret;
-    int rxGlockOwner = ISAFS_RXGLOCK();
     int glockOwner = ISAFS_GLOCK();
 
     tv.tv_sec = msec / 1000;
@@ -542,25 +534,16 @@ afs_osi_Wait(afs_int32 msec, struct afs_osi_WaitHandle *handle, int intok)
 	if (glockOwner) {
 	    AFS_GUNLOCK();
 	}
-	if (rxGlockOwner) {
-	    AFS_RXGUNLOCK();
-	}
 	usr_thread_sleep(&tv);
 	ret = 0;
 	if (glockOwner) {
 	    AFS_GLOCK();
 	}
-	if (rxGlockOwner) {
-	    AFS_RXGLOCK();
-	}
     } else {
 	usr_mutex_lock(&osi_waitq_lock);
 	if (glockOwner) {
 	    AFS_GUNLOCK();
 	}
-	if (rxGlockOwner) {
-	    AFS_RXGUNLOCK();
-	}
 	index = WAITHASH((caddr_t) handle);
 	if (osi_waithash_avail == NULL) {
 	    waitp = (osi_wait_t *) afs_osi_Alloc(sizeof(osi_wait_t));
@@ -593,9 +576,6 @@ afs_osi_Wait(afs_int32 msec, struct afs_osi_WaitHandle *handle, int intok)
 	if (glockOwner) {
 	    AFS_GLOCK();
 	}
-	if (rxGlockOwner) {
-	    AFS_RXGLOCK();
-	}
     }
     return ret;
 }
@@ -874,7 +854,6 @@ afs_osi_Stat(struct osi_file *fp, struct osi_stat *stp)
 	return -1;
     }
     stp->size = st.st_size;
-    stp->blksize = st.st_blksize;
     stp->mtime = st.st_mtime;
     stp->atime = st.st_atime;
     AFS_GLOCK();
@@ -991,10 +970,9 @@ shutdown_osifile(void)
     return;
 }
 
-int
+void
 afs_nfsclient_init(void)
 {
-    return 0;
 }
 
 void
@@ -1411,8 +1389,8 @@ SweepAFSCache(int *vFilesFound)
     return (0);
 }
 
-static
-ConfigCell(register struct afsconf_cell *aci, char *arock,
+static int
+ConfigCell(register struct afsconf_cell *aci, void *arock,
 	   struct afsconf_dir *adir)
 {
     register int isHomeCell;
@@ -1442,10 +1420,7 @@ ConfigCell(register struct afsconf_cell *aci, char *arock,
 }
 
 static int
-ConfigCellAlias(aca, arock, adir)
-	struct afsconf_cellalias *aca;
-	char *arock;
-	struct afsconf_dir *adir;
+ConfigCellAlias(struct afsconf_cellalias *aca, void *arock, struct afsconf_dir *adir)
 {
 	call_syscall(AFSOP_ADDCELLALIAS, (long)aca->aliasName, 
 		     (long)aca->realName, 0, 0, 0);
