@@ -33,7 +33,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/afs_server.c,v 1.33.2.6 2005/08/04 20:05:15 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/afs_server.c,v 1.43.4.3 2007/10/05 02:24:35 shadow Exp $");
 
 #include "afs/stds.h"
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
@@ -488,6 +488,42 @@ afs_CountServers(void)
 }				/*afs_CountServers */
 
 
+void
+ForceAllNewConnections()
+{
+    int srvAddrCount;
+    struct srvAddr **addrs;
+    struct srvAddr *sa;
+    afs_int32 i, j;
+
+    ObtainReadLock(&afs_xserver);	/* Necessary? */
+    ObtainReadLock(&afs_xsrvAddr);
+
+    srvAddrCount = 0;
+    for (i = 0; i < NSERVERS; i++) {
+	for (sa = afs_srvAddrs[i]; sa; sa = sa->next_bkt) {
+	    srvAddrCount++;
+	}
+    }
+
+    addrs = afs_osi_Alloc(srvAddrCount * sizeof(*addrs));
+    j = 0;
+    for (i = 0; i < NSERVERS; i++) {
+	for (sa = afs_srvAddrs[i]; sa; sa = sa->next_bkt) {
+	    if (j >= srvAddrCount)
+		break;
+	    addrs[j++] = sa;
+	}
+    }
+
+    ReleaseReadLock(&afs_xsrvAddr);
+    ReleaseReadLock(&afs_xserver);
+    for (i = 0; i < j; i++) {
+        sa = addrs[i];
+	ForceNewConnections(sa);
+    }
+}
+
 /* check down servers (if adown), or running servers (if !adown) */
 void
 afs_CheckServers(int adown, struct cell *acellp)
@@ -1139,24 +1175,43 @@ afsi_SetServerIPRank(struct srvAddr *sa, struct in_ifaddr *ifa)
 void
 afsi_SetServerIPRank(sa, ifa)
      struct srvAddr *sa;
+#ifdef AFS_DARWIN80_ENV
+     ifaddr_t ifa;
+#else
      struct ifaddr *ifa;
+#endif
 {
+    struct sockaddr sout;
     struct sockaddr_in *sin;
     int t;
 
     afs_uint32 subnetmask, myAddr, myNet, myDstaddr, mySubnet, netMask;
     afs_uint32 serverAddr;
 
-    if (ifa->ifa_addr->sa_family != AF_INET)
+    if (ifaddr_address_family(ifa) != AF_INET)
 	return;
-    sin = (struct sockaddr_in *)ifa->ifa_addr;
-    myAddr = ntohl(sin->sin_addr.s_addr);	/* one of my IP addr in host order */
+    t = ifaddr_address(ifa, &sout, sizeof(sout));
+    if (t == 0) {
+	sin = (struct sockaddr_in *)&sout;
+	myAddr = ntohl(sin->sin_addr.s_addr);	/* one of my IP addr in host order */
+    } else {
+	myAddr = 0;
+    }
     serverAddr = ntohl(sa->sa_ip);	/* server's IP addr in host order */
-    sin = (struct sockaddr_in *)ifa->ifa_netmask;
-    subnetmask = ntohl(sin->sin_addr.s_addr);	/* subnet mask in host order */
-    sin = (struct sockaddr_in *)ifa->ifa_dstaddr;
-    if (sin)
+    t = ifaddr_netmask(ifa, &sout, sizeof(sout));
+    if (t == 0) {
+	sin = (struct sockaddr_in *)&sout;
+	subnetmask = ntohl(sin->sin_addr.s_addr);	/* subnet mask in host order */
+    } else {
+	subnetmask = 0;
+    }
+    t = ifaddr_dstaddress(ifa, &sout, sizeof(sout));
+    if (t == 0) {
+	sin = (struct sockaddr_in *)&sout;
 	myDstaddr = sin->sin_addr.s_addr;
+    } else {
+	myDstaddr = 0;
+    }
 
     if (IN_CLASSA(myAddr))
 	netMask = IN_CLASSA_NET;
@@ -1175,20 +1230,20 @@ afsi_SetServerIPRank(sa, ifa)
 	    if (serverAddr == myAddr) {	/* same machine */
 		sa->sa_iprank = afs_min(sa->sa_iprank, TOPR);
 	    } else {		/* same subnet */
-		sa->sa_iprank = afs_min(sa->sa_iprank, HI + ifa->ifa_metric);
+		sa->sa_iprank = afs_min(sa->sa_iprank, HI + ifnet_metric(ifaddr_ifnet(ifa)));
 	    }
 	} else {		/* same net */
-	    sa->sa_iprank = afs_min(sa->sa_iprank, MED + ifa->ifa_metric);
+	    sa->sa_iprank = afs_min(sa->sa_iprank, MED + ifnet_metric(ifaddr_ifnet(ifa)));
 	}
     }
 #ifdef  IFF_POINTTOPOINT
     /* check for case #4 -- point-to-point link */
-    if ((ifa->ia_ifp->if_flags & IFF_POINTOPOINT)
+    if ((ifnet_flags(ifaddr_ifnet(ifa)) & IFF_POINTOPOINT)
 	&& (myDstaddr == serverAddr)) {
-	if (ifa->ia_ifp->if_metric >= (MAXDEFRANK - MED) / PPWEIGHT)
+	if (ifnet_metric(ifaddr_ifnet(ifa)) >= (MAXDEFRANK - MED) / PPWEIGHT)
 	    t = MAXDEFRANK;
 	else
-	    t = MED + (PPWEIGHT << ifa->->ifa_metric);
+	    t = MED + (PPWEIGHT << ifnet_metric(ifaddr_ifnet(ifa)));
 	if (sa->sa_iprank > t)
 	    sa->sa_iprank = t;
 	}
@@ -1218,23 +1273,74 @@ static int afs_SetServerPrefs(struct srvAddr *sa) {
 #else				/* AFS_USERSPACE_IP_ADDR */
 #if	defined(AFS_SUN5_ENV)
 #ifdef AFS_SUN510_ENV
-    ill_walk_context_t ctx;
+    int i = 0;
 #else
     extern struct ill_s *ill_g_headp;
     long *addr = (long *)ill_g_headp;
-#endif
     ill_t *ill;
     ipif_t *ipif;
+#endif
     int subnet, subnetmask, net, netmask;
 
     if (sa)
 	  sa->sa_iprank = 0;
 #ifdef AFS_SUN510_ENV
-    for (ill = ILL_START_WALK_ALL(&ctx) ; ill ; ill = ill_next(&ctx, ill)) {
+    rw_enter(&afsifinfo_lock, RW_READER);
+
+    for (i = 0; (afsifinfo[i].ipaddr != NULL) && (i < ADDRSPERSITE); i++) {
+
+	if (IN_CLASSA(afsifinfo[i].ipaddr)) {
+	    netmask = IN_CLASSA_NET;
+	} else if (IN_CLASSB(afsifinfo[i].ipaddr)) {
+	    netmask = IN_CLASSB_NET;
+	} else if (IN_CLASSC(afsifinfo[i].ipaddr)) {
+	    netmask = IN_CLASSC_NET;
+	} else {
+	    netmask = 0;
+	}
+	net = afsifinfo[i].ipaddr & netmask;
+
+#ifdef notdef
+	if (!s) {
+	    if (afsifinfo[i].ipaddr != 0x7f000001) {	/* ignore loopback */
+		*cnt += 1;
+		if (*cnt > 16)
+		    return;
+		*addrp++ = afsifinfo[i].ipaddr;
+	    }
+	} else
+#endif /* notdef */
+        {
+            /* XXXXXX Do the individual ip ranking below XXXXX */
+            if ((sa->sa_ip & netmask) == net) {
+                if ((sa->sa_ip & subnetmask) == subnet) {
+                    if (afsifinfo[i].ipaddr == sa->sa_ip) {   /* ie, ME!  */
+                        sa->sa_iprank = TOPR;
+                    } else {
+                        sa->sa_iprank = HI + afsifinfo[i].metric; /* case #2 */
+                    }
+                } else {
+                    sa->sa_iprank = MED + afsifinfo[i].metric;    /* case #3 */
+                }
+            } else {
+                    sa->sa_iprank = LO + afsifinfo[i].metric;     /* case #4 */
+            }
+            /* check for case #5 -- point-to-point link */
+            if ((afsifinfo[i].flags & IFF_POINTOPOINT)
+                && (afsifinfo[i].dstaddr == sa->sa_ip)) {
+
+                    if (afsifinfo[i].metric >= (MAXDEFRANK - MED) / PPWEIGHT)
+                        sa->sa_iprank = MAXDEFRANK;
+                    else
+                        sa->sa_iprank = MED + (PPWEIGHT << afsifinfo[i].metric);
+            }
+        }
+    }
+    
+    rw_exit(&afsifinfo_lock);
 #else
     for (ill = (struct ill_s *)*addr /*ill_g_headp */ ; ill;
 	 ill = ill->ill_next) {
-#endif
 #ifdef AFS_SUN58_ENV
 	/* Make sure this is an IPv4 ILL */
 	if (ill->ill_isv6)
@@ -1294,6 +1400,7 @@ static int afs_SetServerPrefs(struct srvAddr *sa) {
 	    }
 	}
     }
+#endif /* AFS_SUN510_ENV */
 #else
 #ifndef USEIFADDR
     struct ifnet *ifn = NULL;
@@ -1361,6 +1468,27 @@ static int afs_SetServerPrefs(struct srvAddr *sa) {
 #ifdef AFS_SGI62_ENV
     (void)hash_enum(&hashinfo_inaddr, afsi_enum_set_rank, HTF_INET, NULL,
 		    (caddr_t) sa, NULL);
+#elif defined(AFS_DARWIN80_ENV)
+    {
+	errno_t t;
+	unsigned int count;
+	int cnt=0, m, j;
+	ifaddr_t *ifads;
+	ifnet_t *ifn;
+
+	if (!ifnet_list_get(AF_INET, &ifn, &count)) {
+	    for (m = 0; m < count; m++) {
+		if (!ifnet_get_address_list(ifn[m], &ifads)) {
+		    for (j = 0; ifads[j] != NULL && cnt < ADDRSPERSITE; j++) {
+			afsi_SetServerIPRank(sa, ifads[j]);
+			cnt++;
+		    }
+		    ifnet_free_address_list(ifads);
+		}
+	    }
+	    ifnet_list_free(ifn);
+	}
+    }
 #elif defined(AFS_DARWIN60_ENV)
     {
 	struct ifnet *ifn;
@@ -1621,11 +1749,7 @@ struct server *afs_GetServer(afs_uint32 * aserverp, afs_int32 nservers,
 
 	/* Compute preference values and resort */
 	if (!newsa->sa_iprank) {
-	    if (aport == fsport) {
-		afs_SetServerPrefs(newsa);	/* new fileserver rank */
-	    } else {
-		newsa->sa_iprank = 10000 + afs_randomMod127();	/* new vlserver rank */
-	    }
+	    afs_SetServerPrefs(newsa);	/* new server rank */
 	}
     }
     afs_SortOneServer(newts);	/* Sort by rank */
