@@ -11,7 +11,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/bucoord/commands.c,v 1.17 2005/07/11 03:22:19 shadow Exp $");
+    ("$Header: /cvs/openafs/src/bucoord/commands.c,v 1.18.2.5 2007/10/31 04:09:24 shadow Exp $");
 
 #include <afs/stds.h>
 #if defined(AFS_LINUX24_ENV)
@@ -28,7 +28,6 @@ RCSID
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <netdb.h>
-#include <strings.h>
 #endif
 #include <errno.h>
 #include <afs/com_err.h>
@@ -41,7 +40,6 @@ RCSID
 #include <afs/ktime.h>		/* PA */
 #include <time.h>
 
-#include <string.h>
 #include <lock.h>
 #include <afs/butc.h>
 #include <afs/tcdata.h>
@@ -121,7 +119,7 @@ getSPEntries(server, partition, serverlist, ss, ps)
     if (!(*ss)) {
 	*ss = (struct serversort *)malloc(sizeof(struct serversort));
 	if (!(*ss)) {
-	    com_err(whoami, BC_NOMEM, "");
+	    afs_com_err(whoami, BC_NOMEM, "");
 	    *ss = 0;
 	    return (BC_NOMEM);
 	}
@@ -142,7 +140,7 @@ getSPEntries(server, partition, serverlist, ss, ps)
     if (!(*ps)) {
 	*ps = (struct partitionsort *)malloc(sizeof(struct partitionsort));
 	if (!(*ps)) {
-	    com_err(whoami, BC_NOMEM, "");
+	    afs_com_err(whoami, BC_NOMEM, "");
 	    free(*ss);
 	    *ps = 0;
 	    *ss = 0;
@@ -289,7 +287,7 @@ EvalVolumeSet2(aconfig, avs, avols, uclient)
 				 entries[e].serverPartition[ei], &servers,
 				 &ss, &ps);
 		if (tcode) {
-		    com_err(whoami, tcode, "");
+		    afs_com_err(whoami, tcode, "");
 		    ERROR(tcode);
 		}
 
@@ -321,14 +319,14 @@ EvalVolumeSet2(aconfig, avs, avols, uclient)
 		    tvd = (struct bc_volumeDump *)
 			malloc(sizeof(struct bc_volumeDump));
 		    if (!tvd) {
-			com_err(whoami, BC_NOMEM, "");
+			afs_com_err(whoami, BC_NOMEM, "");
 			ERROR(BC_NOMEM);
 		    }
 		    memset(tvd, 0, sizeof(*tvd));
 
 		    tvd->name = (char *)malloc(strlen(entries[e].name) + 10);
 		    if (!(tvd->name)) {
-			com_err(whoami, BC_NOMEM, "");
+			afs_com_err(whoami, BC_NOMEM, "");
 			free(tvd);
 			ERROR(BC_NOMEM);
 		    }
@@ -491,7 +489,7 @@ EvalVolumeSet1(aconfig, avs, avols, uclient)
 		    sprintf(patt, "^%s$", tve->name);
 		    errm = (char *)re_comp(patt);
 		    if (errm) {
-			com_err(whoami, 0,
+			afs_com_err(whoami, 0,
 				"Can't compile regular expression '%s': %s",
 				patt, errm);
 			return (-1);
@@ -545,7 +543,7 @@ EvalVolumeSet1(aconfig, avs, avols, uclient)
 		}
 
 		if (code < 0)
-		    com_err(whoami, 0, "Internal error in regex package");
+		    afs_com_err(whoami, 0, "Internal error in regex package");
 	    }			/*s */
 
 	    /* If found a match, then create a new volume dump entry */
@@ -556,7 +554,7 @@ EvalVolumeSet1(aconfig, avs, avols, uclient)
 				 entry.serverPartition[foundentry], &servers,
 				 &ss, &ps);
 		if (code) {
-		    com_err(whoami, code, "");
+		    afs_com_err(whoami, code, "");
 		    return (code);
 		}
 
@@ -564,14 +562,14 @@ EvalVolumeSet1(aconfig, avs, avols, uclient)
 		tvd = (struct bc_volumeDump *)
 		    malloc(sizeof(struct bc_volumeDump));
 		if (!tvd) {
-		    com_err(whoami, BC_NOMEM, "");
+		    afs_com_err(whoami, BC_NOMEM, "");
 		    return (BC_NOMEM);
 		}
 		memset(tvd, 0, sizeof(*tvd));
 
 		tvd->name = (char *)malloc(strlen(entry.name) + 10);
 		if (!(tvd->name)) {
-		    com_err(whoami, BC_NOMEM, "");
+		    afs_com_err(whoami, BC_NOMEM, "");
 		    free(tvd);
 		    return (BC_NOMEM);
 		}
@@ -714,7 +712,7 @@ bc_CopyString(astring)
     tlen = strlen(astring);
     tp = (char *)malloc(tlen + 1);	/* don't forget the terminating null */
     if (!tp) {
-	com_err(whoami, BC_NOMEM, "");
+	afs_com_err(whoami, BC_NOMEM, "");
 	return (tp);
     }
     strcpy(tp, astring);
@@ -742,13 +740,13 @@ concatParams(itemPtr)
     }
 
     if (length == 0) {		/* no string (0 length) */
-	com_err(whoami, 0, "Can't have zero length date and time string");
+	afs_com_err(whoami, 0, "Can't have zero length date and time string");
 	return (NULL);
     }
 
     string = (char *)malloc(length);	/* allocate the string */
     if (!string) {
-	com_err(whoami, BC_NOMEM, "");
+	afs_com_err(whoami, BC_NOMEM, "");
 	return (NULL);
     }
     string[0] = 0;
@@ -811,10 +809,10 @@ getPortOffset(port)
     portOffset = bc_SafeATOI(port);
 
     if (portOffset < 0) {
-	com_err(whoami, 0, "Can't decode port offset '%s'", port);
+	afs_com_err(whoami, 0, "Can't decode port offset '%s'", port);
 	return (-1);
     } else if (portOffset > BC_MAXPORTOFFSET) {
-	com_err(whoami, 0, "%u exceeds max port offset %u", portOffset,
+	afs_com_err(whoami, 0, "%u exceeds max port offset %u", portOffset,
 		BC_MAXPORTOFFSET);
 	return (-1);
     }
@@ -824,10 +822,8 @@ getPortOffset(port)
 /* bc_GetTapeStatusCmd
  *	display status of all tasks on a particular tape coordinator
  */
-
-bc_GetTapeStatusCmd(as, arock)
-     struct cmd_syndesc *as;
-     char *arock;
+int
+bc_GetTapeStatusCmd(struct cmd_syndesc *as, void *arock)
 {
     afs_int32 code;
     struct rx_connection *tconn;
@@ -840,7 +836,7 @@ bc_GetTapeStatusCmd(as, arock)
 
     code = bc_UpdateHosts();
     if (code) {
-	com_err(whoami, code, "; Can't retrieve tape hosts");
+	afs_com_err(whoami, code, "; Can't retrieve tape hosts");
 	return (code);
     }
 
@@ -863,7 +859,7 @@ bc_GetTapeStatusCmd(as, arock)
 	if (code) {
 	    if (code == TC_NOTASKS)
 		break;
-	    com_err(whoami, code, "; Can't get status from butc");
+	    afs_com_err(whoami, code, "; Can't get status from butc");
 	    return (-1);
 	}
 	if ((flags & TSK_STAT_NOTFOUND))
@@ -899,7 +895,7 @@ bc_WaitForNoJobs()
 
     extern dlqlinkT statusHead;
 
-    com_err(whoami, 0, "waiting for job termination");
+    afs_com_err(whoami, 0, "waiting for job termination");
 
     while (usefulJobRunning) {
 	usefulJobRunning = (dlqEmpty(&statusHead) ? 0 : 1);
@@ -922,10 +918,8 @@ bc_WaitForNoJobs()
  * parameters
  *	ignored - a null "as" prints only jobs.
  */
-
-bc_JobsCmd(as, arock)
-     struct cmd_syndesc *as;
-     char *arock;
+int
+bc_JobsCmd(struct cmd_syndesc *as, void *arock)
 {
     afs_int32 prevTime;
     dlqlinkP ptr;
@@ -1039,9 +1033,8 @@ bc_JobsCmd(as, arock)
     return 0;
 }
 
-bc_KillCmd(as, arock)
-     struct cmd_syndesc *as;
-     char *arock;
+int
+bc_KillCmd(struct cmd_syndesc *as, void *arock)
 {
     afs_int32 i;
     afs_int32 slot;
@@ -1060,7 +1053,7 @@ bc_KillCmd(as, arock)
     if (strchr(tp, '.') == 0) {
 	slot = bc_SafeATOI(tp);
 	if (slot == -1) {
-	    com_err(whoami, 0, "Bad syntax for number '%s'", tp);
+	    afs_com_err(whoami, 0, "Bad syntax for number '%s'", tp);
 	    return -1;
 	}
 
@@ -1093,7 +1086,7 @@ bc_KillCmd(as, arock)
 	    }
 	}
 	if (i >= BC_MAXSIMDUMPS) {
-	    com_err(whoami, 0, "Can't find job %s", tp);
+	    afs_com_err(whoami, 0, "Can't find job %s", tp);
 	    return -1;
 	}
 
@@ -1101,7 +1094,7 @@ bc_KillCmd(as, arock)
 	statusPtr = findStatus(td->dumpID);
 
 	if (statusPtr == 0) {
-	    com_err(whoami, 0, "Can't locate status - internal error");
+	    afs_com_err(whoami, 0, "Can't locate status - internal error");
 	    unlock_Status();
 	    return (-1);
 	}
@@ -1113,9 +1106,8 @@ bc_KillCmd(as, arock)
 }
 
 /* restore a volume or volumes */
-bc_VolRestoreCmd(as, arock)
-     struct cmd_syndesc *as;
-     char *arock;
+int
+bc_VolRestoreCmd(struct cmd_syndesc *as, void *arock)
 {
     /*
      * parm 0 is the new server to restore to
@@ -1141,6 +1133,7 @@ bc_VolRestoreCmd(as, arock)
     afs_int32 code;
     int oldFlag;
     afs_int32 fromDate;
+    afs_int32 dumpID = 0;
     char *newExt, *timeString;
     afs_int32 i;
     afs_int32 *ports = NULL;
@@ -1149,7 +1142,7 @@ bc_VolRestoreCmd(as, arock)
 
     code = bc_UpdateHosts();
     if (code) {
-	com_err(whoami, code, "; Can't retrieve tape hosts");
+	afs_com_err(whoami, code, "; Can't retrieve tape hosts");
 	return (code);
     }
 
@@ -1157,7 +1150,7 @@ bc_VolRestoreCmd(as, arock)
     if (as->parms[0].items) {
 	tp = as->parms[0].items->data;
 	if (bc_ParseHost(tp, &destServ)) {
-	    com_err(whoami, 0, "Failed to locate destination host '%s'", tp);
+	    afs_com_err(whoami, 0, "Failed to locate destination host '%s'", tp);
 	    return -1;
 	}
     }
@@ -1166,7 +1159,7 @@ bc_VolRestoreCmd(as, arock)
     if (as->parms[1].items) {
 	tp = as->parms[1].items->data;
 	if (bc_GetPartitionID(tp, &destPartition)) {
-	    com_err(whoami, 0, "Can't parse destination partition '%s'", tp);
+	    afs_com_err(whoami, 0, "Can't parse destination partition '%s'", tp);
 	    return -1;
 	}
     }
@@ -1175,14 +1168,14 @@ bc_VolRestoreCmd(as, arock)
 	/* build list of volume items */
 	tvol = (struct bc_volumeDump *)malloc(sizeof(struct bc_volumeDump));
 	if (!tvol) {
-	    com_err(whoami, BC_NOMEM, "");
+	    afs_com_err(whoami, BC_NOMEM, "");
 	    return BC_NOMEM;
 	}
 	memset(tvol, 0, sizeof(struct bc_volumeDump));
 
 	tvol->name = (char *)malloc(VOLSER_MAXVOLNAME + 1);
 	if (!tvol->name) {
-	    com_err(whoami, BC_NOMEM, "");
+	    afs_com_err(whoami, BC_NOMEM, "");
 	    return BC_NOMEM;
 	}
 	strncpy(tvol->name, ti->data, VOLSER_OLDMAXVOLNAME);
@@ -1203,8 +1196,8 @@ bc_VolRestoreCmd(as, arock)
 	code = ktime_DateToLong(timeString, &fromDate);
 	free(timeString);
 	if (code) {
-	    com_err(whoami, 0, "Can't parse restore date and time");
-	    com_err(whoami, 0, "%s", ktime_GetDateUsage());
+	    afs_com_err(whoami, 0, "Can't parse restore date and time");
+	    afs_com_err(whoami, 0, "%s", ktime_GetDateUsage());
 	    return code;
 	}
     } else {
@@ -1222,7 +1215,7 @@ bc_VolRestoreCmd(as, arock)
 	    portCount++;
 	ports = (afs_int32 *) malloc(portCount * sizeof(afs_int32));
 	if (!ports) {
-	    com_err(whoami, BC_NOMEM, "");
+	    afs_com_err(whoami, BC_NOMEM, "");
 	    return BC_NOMEM;
 	}
 
@@ -1235,17 +1228,24 @@ bc_VolRestoreCmd(as, arock)
 
     dontExecute = (as->parms[6].items ? 1 : 0);	/* -n */
 
+    if (as->parms[7].items)
+      {
+	dumpID = atoi(as->parms[7].items->data);
+	if (dumpID <= 0)
+	  dumpID = 0;
+      }
+    
     /*
      * Perform the call to start the restore.
      */
     code =
 	bc_StartDmpRst(bc_globalConfig, "volume", "restore", volsToRestore,
 		       &destServ, destPartition, fromDate, newExt, oldFlag,
-		       /*parentDump */ 0, /*dumpLevel */ 0,
+		       /*parentDump */ dumpID, /*dumpLevel */ 0,
 		       bc_Restorer, ports, portCount,
-		       /*dumpSched */ 0, /*append */ 0, dontExecute);
+		       /*dumpSched */ NULL, /*append */ 0, dontExecute);
     if (code)
-	com_err(whoami, code, "; Failed to queue restore");
+	afs_com_err(whoami, code, "; Failed to queue restore");
 
     return (code);
 }
@@ -1260,9 +1260,8 @@ bc_VolRestoreCmd(as, arock)
  *	various optional
  */
 
-bc_DiskRestoreCmd(as, arock)
-     struct cmd_syndesc *as;
-     char *arock;
+int
+bc_DiskRestoreCmd(struct cmd_syndesc *as, void *arock)
 {
     struct bc_volumeSet tvolumeSet;	/* temporary volume set for EvalVolumeSet call */
     struct bc_volumeEntry tvolumeEntry;	/* entry within the volume set */
@@ -1291,12 +1290,12 @@ bc_DiskRestoreCmd(as, arock)
 
     code = bc_UpdateVolumeSet();
     if (code) {
-	com_err(whoami, code, "; Can't retrieve volume sets");
+	afs_com_err(whoami, code, "; Can't retrieve volume sets");
 	return (code);
     }
     code = bc_UpdateHosts();
     if (code) {
-	com_err(whoami, code, "; Can't retrieve tape hosts");
+	afs_com_err(whoami, code, "; Can't retrieve tape hosts");
 	return (code);
     }
 
@@ -1309,13 +1308,13 @@ bc_DiskRestoreCmd(as, arock)
     tvolumeEntry.partname = as->parms[1].items->data;
 
     if (bc_GetPartitionID(tvolumeEntry.partname, &tvolumeEntry.partition)) {
-	com_err(whoami, 0, "Can't parse partition '%s'",
+	afs_com_err(whoami, 0, "Can't parse partition '%s'",
 		tvolumeEntry.partname);
 	return -1;
     }
 
     if (bc_ParseHost(tvolumeEntry.serverName, &tvolumeEntry.server)) {
-	com_err(whoami, 0, "Can't locate host '%s'", tvolumeEntry.serverName);
+	afs_com_err(whoami, 0, "Can't locate host '%s'", tvolumeEntry.serverName);
 	return -1;
     }
 
@@ -1323,7 +1322,7 @@ bc_DiskRestoreCmd(as, arock)
     if (as->parms[8].items) {
 	tp = as->parms[8].items->data;
 	if (bc_ParseHost(tp, &destServ)) {
-	    com_err(whoami, 0, "Can't locate destination host '%s'", tp);
+	    afs_com_err(whoami, 0, "Can't locate destination host '%s'", tp);
 	    return -1;
 	}
     } else			/* use destination host == original host */
@@ -1333,7 +1332,7 @@ bc_DiskRestoreCmd(as, arock)
     if (as->parms[9].items) {
 	tp = as->parms[9].items->data;
 	if (bc_GetPartitionID(tp, &destPartition)) {
-	    com_err(whoami, 0, "Can't parse destination partition '%s'", tp);
+	    afs_com_err(whoami, 0, "Can't parse destination partition '%s'", tp);
 	    return -1;
 	}
     } else			/* use original partition */
@@ -1346,7 +1345,7 @@ bc_DiskRestoreCmd(as, arock)
 	    portCount++;
 	ports = (afs_int32 *) malloc(portCount * sizeof(afs_int32));
 	if (!ports) {
-	    com_err(whoami, BC_NOMEM, "");
+	    afs_com_err(whoami, BC_NOMEM, "");
 	    return BC_NOMEM;
 	}
 
@@ -1367,7 +1366,7 @@ bc_DiskRestoreCmd(as, arock)
 	bc_EvalVolumeSet(bc_globalConfig, &tvolumeSet, &volsToRestore,
 			 cstruct);
     if (code) {
-	com_err(whoami, code, "; Failed to evaluate volume set");
+	afs_com_err(whoami, code, "; Failed to evaluate volume set");
 	return (-1);
     }
 
@@ -1406,9 +1405,9 @@ bc_DiskRestoreCmd(as, arock)
 		       &destServ, destPartition, fromDate, newExt, oldFlag,
 		       /*parentDump */ 0, /*dumpLevel */ 0,
 		       bc_Restorer, ports, portCount,
-		       /*dumpSched */ 0, /*append */ 0, dontExecute);
+		       /*dumpSched */ NULL, /*append */ 0, dontExecute);
     if (code)
-	com_err(whoami, code, "; Failed to queue restore");
+	afs_com_err(whoami, code, "; Failed to queue restore");
 
     return (code);
 }
@@ -1417,9 +1416,8 @@ bc_DiskRestoreCmd(as, arock)
  *	restore a volumeset or list of volumes.
  */
 
-bc_VolsetRestoreCmd(as, arock)
-     struct cmd_syndesc *as;
-     char *arock;
+int
+bc_VolsetRestoreCmd(struct cmd_syndesc *as, void *arock)
 {
     int oldFlag;
     long fromDate;
@@ -1441,25 +1439,25 @@ bc_VolsetRestoreCmd(as, arock)
 
     code = bc_UpdateVolumeSet();
     if (code) {
-	com_err(whoami, code, "; Can't retrieve volume sets");
+	afs_com_err(whoami, code, "; Can't retrieve volume sets");
 	return (code);
     }
     code = bc_UpdateHosts();
     if (code) {
-	com_err(whoami, code, "; Can't retrieve tape hosts");
+	afs_com_err(whoami, code, "; Can't retrieve tape hosts");
 	return (code);
     }
 
     if (as->parms[0].items) {
 	if (as->parms[1].items) {
-	    com_err(whoami, 0, "Can't have both -name and -file options");
+	    afs_com_err(whoami, 0, "Can't have both -name and -file options");
 	    return (-1);
 	}
 
 	volsetName = as->parms[0].items->data;
 	volsetPtr = bc_FindVolumeSet(bc_globalConfig, volsetName);
 	if (!volsetPtr) {
-	    com_err(whoami, 0,
+	    afs_com_err(whoami, 0,
 		    "Can't find volume set '%s' in backup database",
 		    volsetName);
 	    return (-1);
@@ -1470,7 +1468,7 @@ bc_VolsetRestoreCmd(as, arock)
 	    bc_EvalVolumeSet(bc_globalConfig, volsetPtr, &volsToRestore,
 			     cstruct);
 	if (code) {
-	    com_err(whoami, code, "; Failed to evaluate volume set");
+	    afs_com_err(whoami, code, "; Failed to evaluate volume set");
 	    return (-1);
 	}
     } else if (as->parms[1].items) {
@@ -1482,7 +1480,7 @@ bc_VolsetRestoreCmd(as, arock)
 
 	fd = fopen(as->parms[1].items->data, "r");
 	if (!fd) {
-	    com_err(whoami, errno, "; Cannot open file '%s'",
+	    afs_com_err(whoami, errno, "; Cannot open file '%s'",
 		    as->parms[1].items->data);
 	    return (-1);
 	}
@@ -1501,12 +1499,12 @@ bc_VolsetRestoreCmd(as, arock)
 	    }
 
 	    if (bc_ParseHost(server, &destServer)) {
-		com_err(whoami, 0, "Failed to locate host '%s'", server);
+		afs_com_err(whoami, 0, "Failed to locate host '%s'", server);
 		continue;
 	    }
 
 	    if (bc_GetPartitionID(partition, &destPartition)) {
-		com_err(whoami, 0,
+		afs_com_err(whoami, 0,
 			"Failed to parse destination partition '%s'",
 			partition);
 		continue;
@@ -1519,7 +1517,7 @@ bc_VolsetRestoreCmd(as, arock)
 
 	    tvol->name = (char *)malloc(VOLSER_MAXVOLNAME + 1);
 	    if (!tvol->name) {
-		com_err(whoami, BC_NOMEM, "");
+		afs_com_err(whoami, BC_NOMEM, "");
 		return BC_NOMEM;
 	    }
 	    strncpy(tvol->name, volume, VOLSER_OLDMAXVOLNAME);
@@ -1534,7 +1532,7 @@ bc_VolsetRestoreCmd(as, arock)
 	}
 	fclose(fd);
     } else {
-	com_err(whoami, 0, "-name or -file option required");
+	afs_com_err(whoami, 0, "-name or -file option required");
 	return (-1);
     }
 
@@ -1545,7 +1543,7 @@ bc_VolsetRestoreCmd(as, arock)
 	    portCount++;
 	ports = (afs_int32 *) malloc(portCount * sizeof(afs_int32));
 	if (!ports) {
-	    com_err(whoami, BC_NOMEM, "");
+	    afs_com_err(whoami, BC_NOMEM, "");
 	    return BC_NOMEM;
 	}
 
@@ -1564,13 +1562,13 @@ bc_VolsetRestoreCmd(as, arock)
 
     /* Perform the call to start the restore */
     code = bc_StartDmpRst(bc_globalConfig, "disk", "restore", volsToRestore,
-			  /*destserver */ 0, /*destpartition */ 0, fromDate,
+			  /*destserver */ NULL, /*destpartition */ 0, fromDate,
 			  newExt, oldFlag,
 			  /*parentDump */ 0, /*dumpLevel */ 0,
 			  bc_Restorer, ports, portCount,
-			  /*dumpSched */ 0, /*append */ 0, dontExecute);
+			  /*dumpSched */ NULL, /*append */ 0, dontExecute);
     if (code)
-	com_err(whoami, code, "; Failed to queue restore");
+	afs_com_err(whoami, code, "; Failed to queue restore");
 
     return code;
 }
@@ -1600,9 +1598,7 @@ bc_VolsetRestoreCmd(as, arock)
 int dontExecute;
 
 int
-bc_DumpCmd(as, arock)
-     struct cmd_syndesc *as;
-     char *arock;
+bc_DumpCmd(struct cmd_syndesc *as, void *arock)
 {				/*bc_DumpCmd */
     static char rn[] = "bc_DumpCmd";	/*Routine name */
     char *dumpPath, *vsName;	/*Ptrs to various names */
@@ -1633,17 +1629,17 @@ bc_DumpCmd(as, arock)
 
     code = bc_UpdateDumpSchedule();
     if (code) {
-	com_err(whoami, code, "; Can't retrieve dump schedule");
+	afs_com_err(whoami, code, "; Can't retrieve dump schedule");
 	return (code);
     }
     code = bc_UpdateVolumeSet();
     if (code) {
-	com_err(whoami, code, "; Can't retrieve volume sets");
+	afs_com_err(whoami, code, "; Can't retrieve volume sets");
 	return (code);
     }
     code = bc_UpdateHosts();
     if (code) {
-	com_err(whoami, code, "; Can't retrieve tape hosts");
+	afs_com_err(whoami, code, "; Can't retrieve tape hosts");
 	return (code);
     }
 
@@ -1656,13 +1652,13 @@ bc_DumpCmd(as, arock)
 	loadfile = 1;
 	if (as->parms[0].items || as->parms[1].items || as->parms[2].items
 	    || as->parms[4].items) {
-	    com_err(whoami, 0, "Invalid option specified with -file option");
+	    afs_com_err(whoami, 0, "Invalid option specified with -file option");
 	    return -1;
 	}
     } else {
 	loadfile = 0;
 	if (!as->parms[0].items || !as->parms[1].items) {
-	    com_err(whoami, 0,
+	    afs_com_err(whoami, 0,
 		    "Must specify volume set name and dump level name");
 	    return -1;
 	}
@@ -1684,8 +1680,8 @@ bc_DumpCmd(as, arock)
 	code = ktime_DateToLong(timeString, &atTime);
 	free(timeString);
 	if (code) {
-	    com_err(whoami, 0, "Can't parse dump start date and time");
-	    com_err(whoami, 0, "%s", ktime_GetDateUsage());
+	    afs_com_err(whoami, 0, "Can't parse dump start date and time");
+	    afs_com_err(whoami, 0, "%s", ktime_GetDateUsage());
 	    return (1);
 	}
     } else
@@ -1705,7 +1701,7 @@ bc_DumpCmd(as, arock)
 	    portCount = 1;
 	    portp = (afs_int32 *) malloc(sizeof(afs_int32));
 	    if (!portp) {
-		com_err(whoami, BC_NOMEM, "");
+		afs_com_err(whoami, BC_NOMEM, "");
 		return BC_NOMEM;
 	    }
 
@@ -1721,13 +1717,13 @@ bc_DumpCmd(as, arock)
 	 */
 	tvs = bc_FindVolumeSet(bc_globalConfig, vsName);
 	if (!tvs) {
-	    com_err(whoami, 0,
+	    afs_com_err(whoami, 0,
 		    "Can't find volume set '%s' in backup database", vsName);
 	    return (-1);
 	}
 	baseds = bc_FindDumpSchedule(bc_globalConfig, dumpPath);
 	if (!baseds) {
-	    com_err(whoami, 0,
+	    afs_com_err(whoami, 0,
 		    "Can't find dump schedule '%s' in backup database",
 		    dumpPath);
 	    return (-1);
@@ -1747,7 +1743,7 @@ bc_DumpCmd(as, arock)
      */
     if (doAt) {
 	if (atTime < time(0)) {
-	    com_err(whoami, 0,
+	    afs_com_err(whoami, 0,
 		    "Time of dump is earlier then current time - not added");
 	} else {
 	    statusPtr = createStatusNode();
@@ -1786,7 +1782,7 @@ bc_DumpCmd(as, arock)
 	    statusPtr->scheduledDump = atTime;
 	    statusPtr->cmdLine = (char *)malloc(length);
 	    if (!statusPtr->cmdLine) {
-		com_err(whoami, BC_NOMEM, "");
+		afs_com_err(whoami, BC_NOMEM, "");
 		return BC_NOMEM;
 	    }
 
@@ -1820,7 +1816,7 @@ bc_DumpCmd(as, arock)
 
 	    printf("Add scheduled dump as job %d\n", statusPtr->jobNumber);
 	    if ((atTime > ttoken.endTime) && (ttoken.endTime != NEVERDATE))
-		com_err(whoami, 0,
+		afs_com_err(whoami, 0,
 			"Warning: job %d starts after expiration of AFS token",
 			statusPtr->jobNumber);
 
@@ -1838,7 +1834,7 @@ bc_DumpCmd(as, arock)
     if (loadfile) {
 	loadFile = (char *)malloc(strlen(as->parms[6].items->data) + 1);
 	if (!loadFile) {
-	    com_err(whoami, BC_NOMEM, "");
+	    afs_com_err(whoami, BC_NOMEM, "");
 	    return BC_NOMEM;
 	}
 	strcpy(loadFile, as->parms[6].items->data);
@@ -1900,7 +1896,7 @@ bc_DumpCmd(as, arock)
      * dump at the level we requested it be done at.
      */
     if (problemFindingDump) {
-	com_err(whoami, 0,
+	afs_com_err(whoami, 0,
 		"Warning: Doing level %d dump due to missing higher-level dumps",
 		level);
 	if (parent) {
@@ -1913,7 +1909,7 @@ bc_DumpCmd(as, arock)
     /* Expand out the volume set into its component list of volumes. */
     code = bc_EvalVolumeSet(bc_globalConfig, tvs, &volsToDump, cstruct);
     if (code) {
-	com_err(whoami, code, "; Failed to evaluate volume set");
+	afs_com_err(whoami, code, "; Failed to evaluate volume set");
 	return (-1);
     }
     if (!volsToDump) {
@@ -1943,7 +1939,7 @@ bc_DumpCmd(as, arock)
 		    tve->cloneDate = 0;
 
 		if (tve->cloneDate && (tve->cloneDate == tve->date)) {
-		    com_err(whoami, 0,
+		    afs_com_err(whoami, 0,
 			    "Warning: Timestamp on volume %s unchanged from previous dump",
 			    tve->name);
 		}
@@ -1962,13 +1958,13 @@ bc_DumpCmd(as, arock)
 	return (0);
 
     code = bc_StartDmpRst(bc_globalConfig, dumpPath, vsName, volsToDump,
-			  /*destServer */ 0, /*destPartition */ 0,
+			  /*destServer */ NULL, /*destPartition */ 0,
 			  /*fromDate */ 0,
-			  /*newExt */ 0, /*oldFlag */ 0,
+			  /*newExt */ NULL, /*oldFlag */ 0,
 			  parent, level, bc_Dumper, portp, /*portCount */ 1,
 			  baseds, doAppend, dontExecute);
     if (code)
-	com_err(whoami, code, "; Failed to queue dump");
+	afs_com_err(whoami, code, "; Failed to queue dump");
 
     return (code);
 }				/*bc_DumpCmd */
@@ -1980,10 +1976,8 @@ bc_DumpCmd(as, arock)
  * parameters:
  *	ignored
  */
-
-bc_QuitCmd(as, arock)
-     struct cmd_syndesc *as;
-     char *arock;
+int
+bc_QuitCmd(struct cmd_syndesc *as, void *arock)
 {
     int i;
     struct bc_dumpTask *td;
@@ -1998,9 +1992,9 @@ bc_QuitCmd(as, arock)
 	statusPtr = (statusP) ptr;
 	if (!(statusPtr->flags & ABORT_REQUEST)) {
 	    unlock_Status();
-	    com_err(whoami, 0, "Job %d still running (and not aborted)",
+	    afs_com_err(whoami, 0, "Job %d still running (and not aborted)",
 		    statusPtr->jobNumber);
-	    com_err(whoami, 0,
+	    afs_com_err(whoami, 0,
 		    "You must at least 'kill' all running jobs before quitting");
 	    return -1;
 	}
@@ -2012,8 +2006,8 @@ bc_QuitCmd(as, arock)
      */
     for (td = bc_dumpTasks, i = 0; i < BC_MAXSIMDUMPS; i++, td++) {
 	if (td->flags & BC_DI_INUSE) {
-	    com_err(whoami, 0, "A job is still running");
-	    com_err(whoami, 0,
+	    afs_com_err(whoami, 0, "A job is still running");
+	    afs_com_err(whoami, 0,
 		    "You must at least 'kill' all running jobs before quitting");
 	    return -1;
 	}
@@ -2032,10 +2026,8 @@ bc_QuitCmd(as, arock)
  *	Labels a tape i.e. request the tape coordinator to perform this
  *	operation
  */
-
-bc_LabelTapeCmd(as, arock)
-     struct cmd_syndesc *as;
-     char *arock;
+int
+bc_LabelTapeCmd(struct cmd_syndesc *as, void *arock)
 {
     char *tapename = 0, *pname = 0;
     afs_int32 size;
@@ -2044,26 +2036,26 @@ bc_LabelTapeCmd(as, arock)
 
     code = bc_UpdateHosts();
     if (code) {
-	com_err(whoami, code, "; Can't retrieve tape hosts");
+	afs_com_err(whoami, code, "; Can't retrieve tape hosts");
 	return (code);
     }
 
     if (as->parms[0].items) {	/* -name */
 	tapename = as->parms[0].items->data;
 	if (strlen(tapename) >= TC_MAXTAPELEN) {
-	    com_err(whoami, 0, "AFS tape name '%s' is too long", tapename);
+	    afs_com_err(whoami, 0, "AFS tape name '%s' is too long", tapename);
 	    return -1;
 	}
     }
 
     if (as->parms[3].items) {	/* -pname */
 	if (tapename) {
-	    com_err(whoami, 0, "Can only specify -name or -pname");
+	    afs_com_err(whoami, 0, "Can only specify -name or -pname");
 	    return -1;
 	}
 	pname = as->parms[3].items->data;
 	if (strlen(pname) >= TC_MAXTAPELEN) {
-	    com_err(whoami, 0, "Permanent tape name '%s' is too long", pname);
+	    afs_com_err(whoami, 0, "Permanent tape name '%s' is too long", pname);
 	    return -1;
 	}
     }
@@ -2071,7 +2063,7 @@ bc_LabelTapeCmd(as, arock)
     if (as->parms[1].items) {
 	size = bc_FloatATOI(as->parms[1].items->data);
 	if (size == -1) {
-	    com_err(whoami, 0, "Bad syntax for tape size '%s'",
+	    afs_com_err(whoami, 0, "Bad syntax for tape size '%s'",
 		    as->parms[1].items->data);
 	    return -1;
 	}
@@ -2095,17 +2087,15 @@ bc_LabelTapeCmd(as, arock)
  * params:
  *	optional port number
  */
-
-bc_ReadLabelCmd(as, arock)
-     struct cmd_syndesc *as;
-     char *arock;
+int
+bc_ReadLabelCmd(struct cmd_syndesc *as, void *arock)
 {
     afs_int32 code;
     afs_int32 port = 0;
 
     code = bc_UpdateHosts();
     if (code) {
-	com_err(whoami, code, "; Can't retrieve tape hosts");
+	afs_com_err(whoami, code, "; Can't retrieve tape hosts");
 	return (code);
     }
 
@@ -2125,10 +2115,8 @@ bc_ReadLabelCmd(as, arock)
  *	read content information from dump tapes, and if user desires,
  *	add it to the database
  */
-
-bc_ScanDumpsCmd(as, arock)
-     struct cmd_syndesc *as;
-     char *arock;
+int
+bc_ScanDumpsCmd(struct cmd_syndesc *as, void *arock)
 {
     afs_int32 port = 0;
     afs_int32 dbAddFlag = 0;
@@ -2136,7 +2124,7 @@ bc_ScanDumpsCmd(as, arock)
 
     code = bc_UpdateHosts();
     if (code) {
-	com_err(whoami, code, "; Can't retrieve tape hosts");
+	afs_com_err(whoami, code, "; Can't retrieve tape hosts");
 	return (code);
     }
 
@@ -2236,10 +2224,8 @@ bc_ParseExpiration(paramPtr, expType, expDate)
  *	Currently a single option, volumename to search for. Reports
  *	all dumps containing the specified volume
  */
-
-bc_dblookupCmd(as, arock)
-     struct cmd_syndesc *as;
-     char *arock;
+int
+bc_dblookupCmd(struct cmd_syndesc *as, void *arock)
 {
     struct cmd_item *ciptr;
     afs_int32 code;
@@ -2255,10 +2241,8 @@ bc_dblookupCmd(as, arock)
 
 
 /* for ubik version */
-
-bc_dbVerifyCmd(as, arock)
-     struct cmd_syndesc *as;
-     char *arock;
+int
+bc_dbVerifyCmd(struct cmd_syndesc *as, void *arock)
 {
     afs_int32 status;
     afs_int32 orphans;
@@ -2277,7 +2261,7 @@ bc_dbVerifyCmd(as, arock)
 		  &host);
 
     if (code) {
-	com_err(whoami, code, "; Unable to verify database");
+	afs_com_err(whoami, code, "; Unable to verify database");
 	return (-1);
     }
 
@@ -2286,7 +2270,7 @@ bc_dbVerifyCmd(as, arock)
     if (status == 0)
 	printf("Database OK\n");
     else
-	com_err(whoami, status, "; Database is NOT_OK");
+	afs_com_err(whoami, status, "; Database is NOT_OK");
 
     if (detail) {
 	printf("Orphan blocks %d\n", orphans);
@@ -2326,14 +2310,14 @@ deleteDump(dumpid, port, force)
     if (port >= 0) {
 	tcode = bc_UpdateHosts();
 	if (tcode) {
-	    com_err(whoami, tcode, "; Can't retrieve tape hosts");
+	    afs_com_err(whoami, tcode, "; Can't retrieve tape hosts");
 	    ERROR(tcode);
 	}
 
 	/* Find the dump in the backup database */
 	tcode = bcdb_FindDumpByID(dumpid, &dumpEntry);
 	if (tcode) {
-	    com_err(whoami, tcode, "; Unable to locate dumpID %u in database",
+	    afs_com_err(whoami, tcode, "; Unable to locate dumpID %u in database",
 		    dumpid);
 	    ERROR(tcode);
 	}
@@ -2353,7 +2337,7 @@ deleteDump(dumpid, port, force)
 	    if (tcode) {
 		if (tcode == RXGEN_OPCODE)
 		    tcode = BC_VERSIONFAIL;
-		com_err(whoami, tcode,
+		afs_com_err(whoami, tcode,
 			"; Unable to delete dumpID %u via butc", dumpid);
 		ERROR(tcode);
 	    }
@@ -2371,7 +2355,7 @@ deleteDump(dumpid, port, force)
 	    /* Wait for task to finish */
 	    taskflag = waitForTask(taskId);
 	    if (taskflag & (TASK_ERROR | ABORT_DONE)) {
-		com_err(whoami, BUTX_DELETEOBJFAIL,
+		afs_com_err(whoami, BUTX_DELETEOBJFAIL,
 			"; Unable to delete dumpID %u via butc", dumpid);
 		ERROR(BUTX_DELETEOBJFAIL);	/* the task failed */
 	    }
@@ -2391,7 +2375,7 @@ deleteDump(dumpid, port, force)
 
 	tcode = bcdb_deleteDump(dumpid, 0, 0, &dumps);
 	if (tcode) {
-	    com_err(whoami, tcode,
+	    afs_com_err(whoami, tcode,
 		    "; Unable to delete dumpID %u from database", dumpid);
 	    dumps.budb_dumpsList_len = 0;
 	    if (!code)
@@ -2414,10 +2398,8 @@ deleteDump(dumpid, port, force)
  * entry:
  *	dump id - single required arg as param 0.
  */
-
-bc_deleteDumpCmd(as, arock)
-     struct cmd_syndesc *as;
-     char *arock;
+int
+bc_deleteDumpCmd(struct cmd_syndesc *as, void *arock)
 {
     afs_uint32 dumpid;
     afs_int32 code = 0;
@@ -2433,13 +2415,13 @@ bc_deleteDumpCmd(as, arock)
     /* Must specify at least one of -dumpid, -from, or -to */
     if (!as->parms[0].items && !as->parms[1].items && !as->parms[2].items
 	&& !as->parms[4].items) {
-	com_err(whoami, 0, "Must specify at least one field");
+	afs_com_err(whoami, 0, "Must specify at least one field");
 	return (-1);
     }
 
     /* Must have -to option with -from option */
     if (as->parms[1].items && !as->parms[2].items) {
-	com_err(whoami, 0, "Must specify '-to' field with '-from' field");
+	afs_com_err(whoami, 0, "Must specify '-to' field with '-from' field");
 	return (-1);
     }
 
@@ -2455,8 +2437,8 @@ bc_deleteDumpCmd(as, arock)
 	code = ktime_DateToLong(timeString, &fromTime);
 	free(timeString);
 	if (code) {
-	    com_err(whoami, 0, "Can't parse 'from' date and time");
-	    com_err(whoami, 0, "%s", ktime_GetDateUsage());
+	    afs_com_err(whoami, 0, "Can't parse 'from' date and time");
+	    afs_com_err(whoami, 0, "%s", ktime_GetDateUsage());
 	    return (-1);
 	}
 	havetime = 1;
@@ -2487,8 +2469,8 @@ bc_deleteDumpCmd(as, arock)
 	code = ktime_DateToLong(timeString, &toTime);
 	free(timeString);
 	if (code) {
-	    com_err(whoami, 0, "Can't parse 'to' date and time");
-	    com_err(whoami, 0, "%s", ktime_GetDateUsage());
+	    afs_com_err(whoami, 0, "Can't parse 'to' date and time");
+	    afs_com_err(whoami, 0, "%s", ktime_GetDateUsage());
 	    return (-1);
 	}
 	toTime += 59;
@@ -2496,7 +2478,7 @@ bc_deleteDumpCmd(as, arock)
     }
 
     if (fromTime > toTime) {
-	com_err(whoami, 0,
+	afs_com_err(whoami, 0,
 		"'-from' date/time cannot be later than '-to' date/time");
 	return (-1);
     }
@@ -2530,7 +2512,7 @@ bc_deleteDumpCmd(as, arock)
 	code =
 	    bcdb_listDumps(sflags, groupId, fromTime, toTime, &dumps, &flags);
 	if (code) {
-	    com_err(whoami, code,
+	    afs_com_err(whoami, code,
 		    "; Error while deleting dumps from %u to %u", fromTime,
 		    toTime);
 	    rcode = -1;
@@ -2570,9 +2552,8 @@ bc_deleteDumpCmd(as, arock)
     return (rcode);
 }
 
-bc_saveDbCmd(as, arock)
-     struct cmd_syndesc *as;
-     char *arock;
+int
+bc_saveDbCmd(struct cmd_syndesc *as, void *arock)
 {
     struct rx_connection *tconn;
     afs_int32 portOffset = 0;
@@ -2584,7 +2565,7 @@ bc_saveDbCmd(as, arock)
 
     code = bc_UpdateHosts();
     if (code) {
-	com_err(whoami, code, "; Can't retrieve tape hosts");
+	afs_com_err(whoami, code, "; Can't retrieve tape hosts");
 	return (code);
     }
 
@@ -2607,8 +2588,8 @@ bc_saveDbCmd(as, arock)
 	code = ktime_DateToLong(timeString, &toTime);
 	free(timeString);
 	if (code) {
-	    com_err(whoami, 0, "Can't parse '-archive' date and time");
-	    com_err(whoami, 0, "%s", ktime_GetDateUsage());
+	    afs_com_err(whoami, 0, "Can't parse '-archive' date and time");
+	    afs_com_err(whoami, 0, "%s", ktime_GetDateUsage());
 	    return (-1);
 	}
 	toTime += 59;
@@ -2621,7 +2602,7 @@ bc_saveDbCmd(as, arock)
 
     code = TC_SaveDb(tconn, toTime, &taskId);
     if (code) {
-	com_err(whoami, code, "; Failed to save database");
+	afs_com_err(whoami, code, "; Failed to save database");
 	goto exit;
     }
 
@@ -2640,9 +2621,8 @@ bc_saveDbCmd(as, arock)
     return (code);
 }
 
-bc_restoreDbCmd(as, arock)
-     struct cmd_syndesc *as;
-     char *arock;
+int
+bc_restoreDbCmd(struct cmd_syndesc *as, void *arock)
 {
     struct rx_connection *tconn;
     afs_int32 portOffset = 0;
@@ -2652,7 +2632,7 @@ bc_restoreDbCmd(as, arock)
 
     code = bc_UpdateHosts();
     if (code) {
-	com_err(whoami, code, "; Can't retrieve tape hosts");
+	afs_com_err(whoami, code, "; Can't retrieve tape hosts");
 	return (code);
     }
 
@@ -2668,7 +2648,7 @@ bc_restoreDbCmd(as, arock)
 
     code = TC_RestoreDb(tconn, &taskId);
     if (code) {
-	com_err(whoami, code, "; Failed to restore database");
+	afs_com_err(whoami, code, "; Failed to restore database");
 	goto exit;
     }
 
@@ -2835,7 +2815,7 @@ DBLookupByVolume(volumeName)
     }
 
     if (code)
-	com_err(whoami, code, "");
+	afs_com_err(whoami, code, "");
     return (code);
 }
 
@@ -2925,7 +2905,7 @@ dumpInfo(dumpid, detailFlag)
     for (tapeNumber = dumpEntry.tapes.b; tapeNumber <= dumpEntry.tapes.maxTapes; tapeNumber++) {	/*f */
 	tapeLinkPtr = (struct tapeLink *)malloc(sizeof(struct tapeLink));
 	if (!tapeLinkPtr) {
-	    com_err(whoami, BC_NOMEM, "");
+	    afs_com_err(whoami, BC_NOMEM, "");
 	    ERROR(BC_NOMEM);
 	}
 
@@ -2977,7 +2957,7 @@ dumpInfo(dumpid, detailFlag)
 		volumeLinkPtr =
 		    (struct volumeLink *)malloc(sizeof(struct volumeLink));
 		if (!volumeLinkPtr) {
-		    com_err(whoami, BC_NOMEM, "");
+		    afs_com_err(whoami, BC_NOMEM, "");
 		    ERROR(BC_NOMEM);
 		}
 		memset(volumeLinkPtr, 0, sizeof(*volumeLinkPtr));
@@ -3046,7 +3026,7 @@ dumpInfo(dumpid, detailFlag)
 
   error_exit:
     if (code)
-	com_err("dumpInfo", code, "; Can't get dump information");
+	afs_com_err("dumpInfo", code, "; Can't get dump information");
 
     /* free all allocated structures */
     tapeLinkPtr = head;
@@ -3105,7 +3085,7 @@ printRecentDumps(ndumps)
 	if (code) {
 	    if (code == BUDB_ENDOFLIST)
 		return 0;
-	    com_err("dumpInfo", code, "; Can't get dump information");
+	    afs_com_err("dumpInfo", code, "; Can't get dump information");
 	    return (code);
 	}
 
@@ -3145,9 +3125,8 @@ printRecentDumps(ndumps)
  *	as - name of tape
  *	arock -
  */
-bc_dumpInfoCmd(as, arock)
-     struct cmd_syndesc *as;
-     char *arock;
+int
+bc_dumpInfoCmd(struct cmd_syndesc *as, void *arock)
 {
     afs_int32 dumpid;
     afs_int32 detailFlag;
@@ -3158,13 +3137,13 @@ bc_dumpInfoCmd(as, arock)
 
     if (as->parms[0].items) {
 	if (as->parms[1].items) {
-	    com_err(whoami, 0,
+	    afs_com_err(whoami, 0,
 		    "These options are exclusive - select only one");
 	    return (BC_BADARG);
 	}
 	ndumps = atoi(as->parms[0].items->data);
 	if (ndumps <= 0) {
-	    com_err(whoami, 0, "Must provide a positive number");
+	    afs_com_err(whoami, 0, "Must provide a positive number");
 	    return -1;
 	}
 
