@@ -10,6 +10,8 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+RCSID
+    ("$Header: /cvs/openafs/src/budb/server.c,v 1.16.2.6 2007/10/31 04:09:25 shadow Exp $");
 
 #include <fcntl.h>
 #include <sys/stat.h>
@@ -35,7 +37,6 @@
 #include <rx/rxkad.h>
 #include <rx/rx_globals.h>
 #include <afs/cellconfig.h>
-#include <afs/auth.h>
 #include <afs/bubasics.h>
 #include <afs/afsutil.h>
 #include <afs/com_err.h>
@@ -133,7 +134,6 @@ MyBeforeProc(register struct cmd_syndesc *as, void *arock)
  *	initialize all the supported commands and their arguments
  */
 
-void
 initializeArgHandler()
 {
     struct cmd_syndesc *cptr;
@@ -218,9 +218,33 @@ argHandler(struct cmd_syndesc *as, void *arock)
 	ubik_nBuffers = 0;
 
     if (as->parms[7].items != 0) {
+	int tempfd, flags;
+	FILE *auditout;
+	char oldName[MAXPATHLEN];
 	char *fileName = as->parms[7].items->data;
+#ifndef AFS_NT40_ENV
+	struct stat statbuf;
 
-        osi_audit_file(fileName);
+	if ((lstat(fileName, &statbuf) == 0) 
+	    && (S_ISFIFO(statbuf.st_mode))) {
+	    flags = O_WRONLY | O_NONBLOCK;
+	} else 
+#endif
+	{
+	    strcpy(oldName, fileName);
+	    strcat(oldName, ".old");
+	    renamefile(fileName, oldName);
+	    flags = O_WRONLY | O_TRUNC | O_CREAT;
+	}
+	tempfd = open(fileName, flags, 0666);
+	if (tempfd > -1) {
+	    auditout = fdopen(tempfd, "a");
+	    if (auditout) {
+		osi_audit_file(auditout);
+	    } else
+		printf("Warning: auditlog %s not writable, ignored.\n", fileName);
+	} else
+	    printf("Warning: auditlog %s not writable, ignored.\n", fileName);
     }
 
     return 0;
@@ -509,7 +533,7 @@ main(argc, argv)
     code = ubik_ServerInitByInfo (globalConfPtr->myHost,
 				  htons(AFSCONF_BUDBPORT), 
 				  &cellinfo,
-				  clones,              
+				  &clones,              
 				  dbNamePtr,           /* name prefix */
 				  &BU_dbase);
 
@@ -524,10 +548,6 @@ main(argc, argv)
     sca[RX_SCINDEX_KAD] =
 	rxkad_NewServerSecurityObject(rxkad_clear, BU_conf, afsconf_GetKey,
 				      NULL);
-#ifdef USE_RXKAD_KEYTAB
-    if (rxkad_InitKeytabDecrypt(AFSDIR_SERVER_RXKAD_KEYTAB_FILEPATH) == 0)
-        rxkad_BindKeytabDecrypt(sca[RX_SCINDEX_KAD]);
-#endif
 
     /* Disable jumbograms */
     rx_SetNoJumbo();
@@ -563,14 +583,14 @@ main(argc, argv)
     currentTime = time(0);
     LogError(0, "Ready to process requests at %s\n", ctime(&currentTime));
 
-    rx_ServerProc(NULL);		/* donate this LWP */
+    rx_ServerProc();		/* donate this LWP */
 
   error_exit:
     osi_audit(BUDB_FinishEvent, code, AUD_END);
     return (code);
 }
 
-void
+
 consistencyCheckDb()
 {
     /* do consistency checks on structure sizes */
