@@ -12,20 +12,14 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/gtx/frame.c,v 1.7 2003/07/15 23:15:12 shadow Exp $");
+    ("$Header: /cvs/openafs/src/gtx/frame.c,v 1.7.14.1 2007/10/30 15:16:39 shadow Exp $");
 
 #ifdef AFS_HPUX_ENV
 #include <sys/types.h>
 #endif
 #include <lwp.h>
 
-#ifdef HAVE_STRING_H
 #include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
 #include <stdlib.h>
 
 #include "gtxobjects.h"
