@@ -17,7 +17,7 @@
 #endif
 
 RCSID
-    ("$Header: /cvs/openafs/src/kauth/authclient.c,v 1.14.2.3 2004/12/07 06:07:20 shadow Exp $");
+    ("$Header: /cvs/openafs/src/kauth/authclient.c,v 1.18.8.3 2007/10/30 15:16:39 shadow Exp $");
 
 #if defined(UKERNEL)
 #include "afs/sysincludes.h"
@@ -46,13 +46,7 @@ RCSID
 #include <sys/socket.h>
 #include <netinet/in.h>
 #endif
-#ifdef HAVE_STRING_H
 #include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
 #include <rx/rxkad.h>
 #include <afs/cellconfig.h>
 #include <ubik.h>
@@ -475,8 +469,10 @@ kawrap_ubik_Call(aproc, aclient, aflags, p1, p2, p3, p4, p5, p6, p7, p8)
 	do {			/* Cycle through the servers */
 	    lcode = code;
 	    code =
-		ubik_CallIter(aproc, aclient, aflags, &count, p1, p2, p3, p4,
-			      p5, p6, p7, p8, 0, 0, 0, 0, 0, 0, 0, 0);
+		ubik_CallIter(aproc, aclient, aflags, &count, (long) p1,
+			      (long) p2, (long) p3, (long) p4,
+			      (long) p5, (long) p6, (long) p7,
+			      (long) p8, 0, 0, 0, 0, 0, 0, 0, 0);
 	} while ((code == UNOQUORUM) || (code == UNOTSYNC)
 		 || (code == KALOCKED) || (code == -1));
 
