@@ -11,7 +11,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/kauth/kadatabase.c,v 1.10 2003/12/07 22:49:26 jaltman Exp $");
+    ("$Header: /cvs/openafs/src/kauth/kadatabase.c,v 1.11.8.1 2007/10/30 15:16:39 shadow Exp $");
 
 #include <sys/types.h>
 #ifdef AFS_NT40_ENV
@@ -19,14 +19,7 @@ RCSID
 #else
 #include <netinet/in.h>
 #endif
-#ifdef HAVE_STRING_H
 #include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
-#include <lock.h>
 #include <ubik.h>
 #include <rx/xdr.h>
 #include <rx/rx.h>
@@ -689,11 +682,12 @@ ka_debugKeyCache(info)
 		    (keyCache[i].superseded == NEVERDATE);
 		info->kcInfo[j].keycksum = 0;
 #if DEBUG_KEY_CACHE
-        {
-        int k;
-		for (k = 0; k < sizeof(struct ktc_encryptionKey); k++)
-		    info->kcInfo[j].keycksum += ((char *)&keyCache[i].key)[k];
-        }
+		{
+		    int k;
+		    for (k = 0; k < sizeof(struct ktc_encryptionKey); k++)
+			info->kcInfo[j].keycksum +=
+			    ((char *)&keyCache[i].key)[k];
+		}
 #endif
 		strcpy(principal, keyCache[i].name);
 		strcat(principal, ".");
