@@ -11,7 +11,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/kauth/kpwvalid.c,v 1.6.2.3 2008/10/27 23:54:09 shadow Exp $");
+    ("$Header: /cvs/openafs/src/kauth/kpwvalid.c,v 1.6.14.1 2007/10/30 15:16:39 shadow Exp $");
 
 #include <stdio.h>
 #include <string.h>
@@ -27,7 +27,7 @@ main(int argc, char *argv[])
 {
     char oldpassword[512];
     char password[512];
-    int rc = 1;
+    int rc;
 
     if (fgets(oldpassword, 512, stdin))
 	while (fgets(password, 512, stdin)) {
@@ -43,5 +43,4 @@ main(int argc, char *argv[])
 		fflush(stdout);
 	    }
 	}
-    return rc;
 }
