@@ -11,7 +11,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/kauth/rebuild.c,v 1.13 2006/04/04 20:51:16 shadow Exp $");
+    ("$Header: /cvs/openafs/src/kauth/rebuild.c,v 1.13.2.3 2007/10/31 04:09:30 shadow Exp $");
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -23,13 +23,7 @@ RCSID
 #include <sys/file.h>
 #include <netinet/in.h>
 #endif
-#ifdef HAVE_STRING_H
 #include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
 #include <stdio.h>
 #include <errno.h>
 #include <time.h>
@@ -394,21 +388,20 @@ readDB(offset, buffer, size)
     offset += UBIK_HEADERSIZE;
     code = lseek(fd, offset, SEEK_SET);
     if (code != offset) {
-	com_err(whoami, errno, "skipping Ubik header");
+	afs_com_err(whoami, errno, "skipping Ubik header");
 	exit(2);
     }
     code = read(fd, buffer, size);
     if (code != size) {
-	com_err(whoami, errno, "reading db got %d bytes", code);
+	afs_com_err(whoami, errno, "reading db got %d bytes", code);
 	exit(3);
     }
 }
 
 #include "AFS_component_version_number.c"
 
-WorkerBee(as, arock)
-     struct cmd_syndesc *as;
-     char *arock;
+static int
+WorkerBee(struct cmd_syndesc *as, void *arock)
 {
     afs_int32 code;
     char *dbFile;
@@ -430,7 +423,7 @@ WorkerBee(as, arock)
     if (outFile) {
 	out = fopen(outFile, "w");
 	if (!out) {
-	    com_err(whoami, errno, "opening output file %s", outFile);
+	    afs_com_err(whoami, errno, "opening output file %s", outFile);
 	    exit(7);
 	}
     } else
@@ -438,12 +431,12 @@ WorkerBee(as, arock)
 
     fd = open(dbFile, O_RDONLY, 0);
     if (fd < 0) {
-	com_err(whoami, errno, "opening database file %s", dbFile);
+	afs_com_err(whoami, errno, "opening database file %s", dbFile);
 	exit(6);
     }
     code = fstat(fd, &info);
     if (code) {
-	com_err(whoami, errno, "stat'ing file %s", dbFile);
+	afs_com_err(whoami, errno, "stat'ing file %s", dbFile);
 	exit(6);
     }
     if ((info.st_size - UBIK_HEADERSIZE) % UBIK_BUFFERSIZE)
