@@ -17,7 +17,7 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID("$Header: /cvs/openafs/src/lwp/lwp.c,v 1.27.2.2 2004/12/13 19:40:19 shadow Exp $");
+RCSID("$Header: /cvs/openafs/src/lwp/lwp.c,v 1.34.2.2 2007/10/30 15:16:42 shadow Exp $");
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -33,18 +33,13 @@ RCSID("$Header: /cvs/openafs/src/lwp/lwp.c,v 1.27.2.2 2004/12/13 19:40:19 shadow
 #include <sys/pseg.h>
 #include <sys/core.h>
 #pragma alloca
+int setlim(int limcon, uchar_t hard, int limit);
 #endif
 #ifdef AFS_SGI64_ENV
 extern char *getenv();
 #include <time.h>
 #endif
-#ifdef HAVE_STRING_H
 #include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
 
 #if	!defined(USE_PTHREADS) && !defined(USE_SOLARIS_THREADS)
 
@@ -80,6 +75,8 @@ extern char PRE_Block;		/* from preempt.c */
 #ifdef __s390__
 #define MINFRAME    96
 #define STACK_ALIGN 8
+#elif defined(AFS_DARWIN_ENV)
+#define STACK_ALIGN 16
 #else
 #define STACK_ALIGN 4
 #endif
@@ -121,7 +118,7 @@ static purge_dead_pcbs();
 struct QUEUE {
     PROCESS head;
     int count;
-} runnable[MAX_PRIORITIES], blocked;
+} runnable[MAX_PRIORITIES], blocked, qwaiting;
 /* Invariant for runnable queues: The head of each queue points to the currently running process if it is in that queue, or it points to the next process in that queue that should run. */
 
 /* Offset of stack field within pcb -- used by stack checking stuff */
@@ -145,9 +142,7 @@ int lwp_nextindex;
 int lwp_MinStackSize = 0;
 
 static int
-lwp_remove(p, q)
-     register PROCESS p;
-     register struct QUEUE *q;
+lwp_remove(register PROCESS p, register struct QUEUE *q)
 {
     /* Special test for only element on queue */
     if (q->count == 1)
@@ -166,9 +161,7 @@ lwp_remove(p, q)
 }
 
 static int
-insert(p, q)
-     register PROCESS p;
-     register struct QUEUE *q;
+insert(register PROCESS p, register struct QUEUE *q)
 {
     if (q->head == NULL) {	/* Queue is empty */
 	q->head = p;
@@ -184,9 +177,7 @@ insert(p, q)
 }
 
 static int
-move(p, from, to)
-     PROCESS p;
-     struct QUEUE *from, *to;
+move(PROCESS p, struct QUEUE *from, struct QUEUE *to)
 {
 
     lwp_remove(p, from);
@@ -242,18 +233,17 @@ LWP_QWait(void)
 {
     register PROCESS tp;
     (tp = lwp_cpptr)->status = QWAITING;
-    lwp_remove(tp, &runnable[tp->priority]);
+    move(tp, &runnable[tp->priority], &qwaiting);
     Set_LWP_RC();
     return LWP_SUCCESS;
 }
 
 int
-LWP_QSignal(pid)
-     register PROCESS pid;
+LWP_QSignal(register PROCESS pid)
 {
     if (pid->status == QWAITING) {
 	pid->status = READY;
-	insert(pid, &runnable[pid->priority]);
+	move(pid, &qwaiting, &runnable[pid->priority]);
 	return LWP_SUCCESS;
     } else
 	return LWP_ENOWAIT;
@@ -279,6 +269,7 @@ LWP_CreateProcess(int (*ep) (), int stacksize, int priority, void *parm,
 #else
     char *stackptr;
 #endif
+    char *stackmemory;
 
 #if defined(AFS_LWP_MINSTACKSIZE)
     /*
@@ -303,7 +294,11 @@ LWP_CreateProcess(int (*ep) (), int stacksize, int priority, void *parm,
 	    return LWP_ENOMEM;
 	}
 	if (stacksize < MINSTACK)
+#ifdef AFS_DARWIN_ENV
+	    stacksize = 1008;
+#else /* !AFS_DARWIN_ENV */
 	    stacksize = 1000;
+#endif /* !AFS_DARWIN_ENV */
 	else
 	    stacksize =
 		STACK_ALIGN * ((stacksize + STACK_ALIGN - 1) / STACK_ALIGN);
@@ -346,20 +341,30 @@ LWP_CreateProcess(int (*ep) (), int stacksize, int priority, void *parm,
 	    stackptr = reserveFromStack(lwp_MaxStackSize);
 	}
 	stackptr -= stacksize;
+	stackmemory = stackptr;
 #else
-	if ((stackptr = (char *)malloc(stacksize + 7)) == NULL) {
+#ifdef AFS_DARWIN_ENV
+	if ((stackmemory = (char *)malloc(stacksize + STACK_ALIGN - 1)) == NULL)
+#else /* !AFS_DARWIN_ENV */
+	if ((stackmemory = (char *)malloc(stacksize + 7)) == NULL)
+#endif /* !AFS_DARWIN_ENV */
+	{
 	    Set_LWP_RC();
 	    return LWP_ENOMEM;
 	}
 	/* Round stack pointer to byte boundary */
-	stackptr = (char *)(8 * (((long)stackptr + 7) / 8));
+#ifdef AFS_DARWIN_ENV
+	stackptr = (char *)(STACK_ALIGN * (((long)stackmemory + STACK_ALIGN - 1) / STACK_ALIGN));
+#else /* !AFS_DARWIN_ENV */
+	stackptr = (char *)(8 * (((long)stackmemory + 7) / 8));
+#endif /* !AFS_DARWIN_ENV */
 #endif
 	if (priority < 0 || priority >= MAX_PRIORITIES) {
 	    Set_LWP_RC();
 	    return LWP_EBADPRI;
 	}
 	Initialize_Stack(stackptr, stacksize);
-	Initialize_PCB(temp, priority, stackptr, stacksize, ep, parm, name);
+	Initialize_PCB(temp, priority, stackmemory, stacksize, ep, parm, name);
 	insert(temp, &runnable[priority]);
 	temp2 = lwp_cpptr;
 	if (PRE_Block != 0)
@@ -377,8 +382,12 @@ LWP_CreateProcess(int (*ep) (), int stacksize, int priority, void *parm,
 		    stackptr + MINFRAME);
 #else
 #if defined(AFS_SGI62_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_FBSD_ENV)
+#ifdef sys_x86_darwin_80
+	savecontext(Create_Process_Part2, &temp2->context, stackptr + stacksize - 16 - sizeof(void *));	/* 16 = 2 * jmp_buf_type */
+#else /* !sys_x86_darwin_80 */
 	/* Need to have the sp on an 8-byte boundary for storing doubles. */
 	savecontext(Create_Process_Part2, &temp2->context, stackptr + stacksize - 16);	/* 16 = 2 * jmp_buf_type */
+#endif /* !sys_x86_darwin_80 */
 #else
 #if defined(AFS_SPARC64_LINUX20_ENV) || defined(AFS_SPARC_LINUX20_ENV)
 	savecontext(Create_Process_Part2, &temp2->context, stackptr + stacksize - 0x40);	/* lomgjmp does something
@@ -561,6 +570,9 @@ Dump_Processes(void)
 	    for_all_elts(x, blocked, {
 			 Dump_One_Process(x);}
 	)
+	    for_all_elts(x, qwaiting, {
+			 Dump_One_Process(x);}
+	)
     } else
 	printf("***LWP: LWP support not initialized\n");
     return 0;
@@ -601,6 +613,8 @@ LWP_InitializeProcessSupport(int priority, PROCESS * pid)
     }
     blocked.head = NULL;
     blocked.count = 0;
+    qwaiting.head = NULL;
+    qwaiting.count = 0;
     lwp_init = (struct lwp_ctl *)malloc(sizeof(struct lwp_ctl));
     temp = (PROCESS) malloc(sizeof(struct lwp_pcb));
     if (lwp_init == NULL || temp == NULL)
@@ -659,6 +673,9 @@ LWP_TerminateProcessSupport(void)
     )
 	for_all_elts(cur, blocked, {
 		     Free_PCB(cur);}
+    )
+	for_all_elts(cur, qwaiting, {
+		     Free_PCB(cur);}
     )
 	free(lwp_init);
     lwp_init = NULL;
@@ -784,7 +801,9 @@ Delete_PCB(register PROCESS pid)
     lwp_remove(pid,
 	       (pid->blockflag || pid->status == WAITING
 		|| pid->status ==
-		DESTROYED ? &blocked : &runnable[pid->priority]));
+		DESTROYED ? &blocked : 
+		(pid->status == QWAITING) ? &qwaiting :
+		&runnable[pid->priority]));
     LWPANCHOR.processcnt--;
     return 0;
 }
@@ -811,6 +830,9 @@ Dump_One_Process(PROCESS pid)
     case DESTROYED:
 	printf("DESTROYED");
 	break;
+    case QWAITING:
+	printf("QWAITING");
+	break;
     default:
 	printf("unknown");
     }
@@ -822,7 +844,6 @@ Dump_One_Process(PROCESS pid)
 	       pid->stacksize, pid->stack);
 	printf("***LWP: HWM stack usage: ");
 	printf("%d\n", Stack_Used(pid->stack, pid->stacksize));
-	free(pid->stack);
     }
     printf("***LWP: Current Stack Pointer: 0x%x\n", pid->context.topstack);
     if (pid->eventcnt > 0) {
@@ -870,7 +891,13 @@ Dispatcher(void)
 		     printf(" \"%s\"", p->name);
 		     }
 	)
-	    puts("]");
+	puts("]");
+	printf("[Qwaiting (%d):", qwaiting.count);
+	for_all_elts(p, qwaiting, {
+		     printf(" \"%s\"", p->name);
+		     }
+	)
+	puts("]");
     }
 #endif
 
@@ -980,7 +1007,9 @@ Free_PCB(PROCESS pid)
 	Debug(0,
 	      ("HWM stack usage: %d, [PCB at 0x%x]",
 	       Stack_Used(pid->stack, pid->stacksize), pid));
+#ifndef AFS_AIX32_ENV
 	free(pid->stack);
+#endif
     }
     if (pid->eventlist != NULL)
 	free(pid->eventlist);
