@@ -7,23 +7,11 @@
  * directory or online at http://www.openafs.org/dl/license10.html
  */
 
-/*
- *                      (3) add new pts commands:
- *
- *                          Interactive - allow the pts command
- *                                        to be run interactively.
- *                          Quit        - quit interactive mode.
- *                          Source      - allow input to come from a file(s).
- *                          Sleep       - pause for a specified number
- *                                        of seconds.
- *
- */
-
 #include <afsconfig.h>
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/ptserver/pts.c,v 1.15 2006/03/13 23:15:44 rees Exp $");
+    ("$Header: /cvs/openafs/src/ptserver/pts.c,v 1.15.2.3 2007/10/31 04:09:32 shadow Exp $");
 
 #include <stdio.h>
 #include <string.h>
@@ -46,23 +34,13 @@ RCSID
 #include "ptclient.h"
 #include "pterror.h"
 #include <afs/afsutil.h>
+#include <afs/com_err.h>
 
 #undef FOREIGN
 
 char *whoami;
 int force = 0;
 
-#if defined(SUPERGROUPS)
-
-/*
- *  Add new pts commands:
- *
- *      Interactive - allow the pts command to be run interactively.
- *      Quit        - quit interactive mode.
- *      Source      - allow input to come from a file(s).
- *      Sleep       - pause for a specified number of seconds.
- */
-
 static int finished;
 static FILE *source;
 extern struct ubik_client *pruclient;
@@ -73,21 +51,21 @@ struct sourcestack {
 } *shead;
 
 int
-Interactive(struct cmd_syndesc *as, char *arock)
+pts_Interactive(struct cmd_syndesc *as, void *arock)
 {
     finished = 0;
     return 0;
 }
 
 int
-Quit(struct cmd_syndesc *as, char *arock)
+pts_Quit(struct cmd_syndesc *as, void *arock)
 {
     finished = 1;
     return 0;
 }
 
 int
-Source(struct cmd_syndesc *as, char *arock)
+pts_Source(struct cmd_syndesc *as, void *arock)
 {
     FILE *fd;
     struct sourcestack *sp;
@@ -115,7 +93,7 @@ Source(struct cmd_syndesc *as, char *arock)
 }
 
 int
-Sleep(struct cmd_syndesc *as, char *arock)
+pts_Sleep(struct cmd_syndesc *as, void *arock)
 {
     int delay;
     if (!as->parms[0].items) {
@@ -124,6 +102,7 @@ Sleep(struct cmd_syndesc *as, char *arock)
     }
     delay = atoi(as->parms[0].items->data);
     IOMGR_Sleep(delay);
+    return 0;
 }
 
 int
@@ -140,8 +119,6 @@ popsource()
     return 1;
 }
 
-#endif /* SUPERGROUPS */
-
 int
 osi_audit()
 {
@@ -153,7 +130,7 @@ osi_audit()
 }
 
 int
-GetGlobals(struct cmd_syndesc *as, char *arock)
+GetGlobals(struct cmd_syndesc *as, void *arock)
 {
     register afs_int32 code;
     char *cell;
@@ -176,7 +153,7 @@ GetGlobals(struct cmd_syndesc *as, char *arock)
 	code = pr_Initialize(sec, AFSDIR_CLIENT_ETC_DIRPATH, cell);
     }
     if (code) {
-	com_err(whoami, code, "while initializing");
+	afs_com_err(whoami, code, "while initializing");
 	return code;
     }
     if (as->parms[19].items)
@@ -185,9 +162,8 @@ GetGlobals(struct cmd_syndesc *as, char *arock)
 }
 
 int
-CleanUp(struct cmd_syndesc *as, char *arock)
+CleanUp(struct cmd_syndesc *as, void *arock)
 {
-#if defined(SUPERGROUPS)
     if (as && !strcmp(as->name, "help"))
 	return 0;
     if (pruclient) {
@@ -195,19 +171,11 @@ CleanUp(struct cmd_syndesc *as, char *arock)
 	pr_End();
 	rx_Finalize();
     }
-#else
-    if (!strcmp(as->name, "help"))
-	return 0;
-    /* Need to shutdown the ubik_client & other connections */
-    pr_End();
-    rx_Finalize();
-#endif /* SUPERGROUPS */
-
     return 0;
 }
 
 int
-CreateGroup(struct cmd_syndesc *as, char *arock)
+CreateGroup(struct cmd_syndesc *as, void *arock)
 {
     register afs_int32 code;
     afs_int32 id;
@@ -226,22 +194,22 @@ CreateGroup(struct cmd_syndesc *as, char *arock)
 	if (idi) {
 	    code = util_GetInt32(idi->data, &id);
 	    if (code) {
-		com_err(whoami, code, "because group id was: '%s'",
+		afs_com_err(whoami, code, "because group id was: '%s'",
 			idi->data);
 		return code;
 	    }
 	    if (id >= 0) {
 		code = PRBADARG;
-		com_err(whoami, code, "because group id %d was not negative",
+		afs_com_err(whoami, code, "because group id %d was not negative",
 			id);
 		return code;
 	    }
-#if defined(SUPERGROUPS)
-	    if (id == 0) {
+	    
+            if (id == 0) {
 		printf("0 isn't a valid user id; aborting\n");
 		return EINVAL;
 	    }
-#endif
+
 	    idi = idi->next;
 	} else
 	    id = 0;
@@ -249,13 +217,13 @@ CreateGroup(struct cmd_syndesc *as, char *arock)
 	code = pr_CreateGroup(namei->data, owner, &id);
 	if (code) {
 	    if (owner || id)
-		com_err(whoami, code,
+		afs_com_err(whoami, code,
 			"; unable to create group %s with id %d%s%s%s%s",
 			namei->data, id, owner ? " owned by '" : "",
 			owner ? owner : "", owner ? "'" : "",
 			(force ? " (ignored)" : ""));
 	    else
-		com_err(whoami, code, "; unable to create group %s %s",
+		afs_com_err(whoami, code, "; unable to create group %s %s",
 			namei->data, (force ? "(ignored)" : ""));
 	    if (!force)
 		return code;
@@ -267,7 +235,7 @@ CreateGroup(struct cmd_syndesc *as, char *arock)
 }
 
 int
-CreateUser(struct cmd_syndesc *as, char *arock)
+CreateUser(struct cmd_syndesc *as, void *arock)
 {
     register afs_int32 code;
     afs_int32 id;
@@ -281,7 +249,7 @@ CreateUser(struct cmd_syndesc *as, char *arock)
 	if (idi) {
 	    code = util_GetInt32(idi->data, &id);
 	    if (code) {
-		com_err(whoami, code, "because id was: '%s'", idi->data);
+		afs_com_err(whoami, code, "because id was: '%s'", idi->data);
 		return code;
 	    }
 	    if (id == 0) {
@@ -295,11 +263,11 @@ CreateUser(struct cmd_syndesc *as, char *arock)
 	code = pr_CreateUser(namei->data, &id);
 	if (code) {
 	    if (id)
-		com_err(whoami, code,
+		afs_com_err(whoami, code,
 			"; unable to create user %s with id %d %s",
 			namei->data, id, (force ? "(ignored)" : ""));
 	    else
-		com_err(whoami, code, "; unable to create user %s %s",
+		afs_com_err(whoami, code, "; unable to create user %s %s",
 			namei->data, (force ? "(ignored)" : ""));
 	    if (!force)
 		return code;
@@ -321,11 +289,11 @@ GetNameOrId(register struct cmd_syndesc *as, struct idlist *lids, struct namelis
     int goodCount;
 
     if (!(as->parms[0].items || as->parms[1].items)) {
-	com_err(whoami, 0, "must specify either a name or an id.");
+	afs_com_err(whoami, 0, "must specify either a name or an id.");
 	return -1;
     }
     if (as->parms[0].items && as->parms[1].items) {
-	com_err(whoami, 0, "can't specify both a name and id.");
+	afs_com_err(whoami, 0, "can't specify both a name and id.");
 	return -1;
     }
 
@@ -354,11 +322,11 @@ GetNameOrId(register struct cmd_syndesc *as, struct idlist *lids, struct namelis
 
 	code = pr_NameToId(nl, lids);
 	if (code)
-	    com_err(whoami, code, "so couldn't look up names");
+	    afs_com_err(whoami, code, "so couldn't look up names");
 	else {
 	    for (n = 0; n < lids->idlist_len; n++) {
 		if ((lids->idlist_val[n] == ANONYMOUSID)) {
-		    com_err(whoami, PRNOENT, "so couldn't look up id for %s",
+		    afs_com_err(whoami, PRNOENT, "so couldn't look up id for %s",
 			    nl->namelist_val[n]);
 		} else
 		    goodCount++;
@@ -380,7 +348,7 @@ GetNameOrId(register struct cmd_syndesc *as, struct idlist *lids, struct namelis
 	for (i = as->parms[1].items; i; i = i->next) {
 	    code = util_GetInt32(i->data, &lids->idlist_val[n]);
 	    if (code)
-		com_err(whoami, code =
+		afs_com_err(whoami, code =
 			PRNOENT, "because a bogus id '%s' was specified",
 			i->data);
 	    n++;
@@ -390,7 +358,7 @@ GetNameOrId(register struct cmd_syndesc *as, struct idlist *lids, struct namelis
 	    lnames->namelist_len = 0;
 	    code = pr_IdToName(lids, lnames);
 	    if (code)
-		com_err(whoami, code, "translating ids");
+		afs_com_err(whoami, code, "translating ids");
 	}
     }
     if (code) {
@@ -447,11 +415,11 @@ GetNameOrId(register struct cmd_syndesc *as, struct idlist *lids, struct namelis
     tids.idlist_val = 0;
     code = pr_NameToId(&names, &tids);
     if (code)
-	com_err(whoami, code, "so couldn't look up names");
+	afs_com_err(whoami, code, "so couldn't look up names");
     else {
 	for (n = 0; n < tids.idlist_len; n++) {
 	    if ((tids.idlist_val[n] == ANONYMOUSID)) {
-		com_err(whoami, PRNOENT, "so couldn't look up id for %s",
+		afs_com_err(whoami, PRNOENT, "so couldn't look up id for %s",
 			names.namelist_val[n]);
 	    } else
 		goodCount++;
@@ -470,7 +438,7 @@ GetNameOrId(register struct cmd_syndesc *as, struct idlist *lids, struct namelis
 	tnames.namelist_len = 0;
 	code = pr_IdToName(&ids, &tnames);
 	if (code)
-	    com_err(whoami, code, "translating ids");
+	    afs_com_err(whoami, code, "translating ids");
 	else
 	    goodCount++;
 	if (lnames) {
@@ -492,7 +460,7 @@ GetNameOrId(register struct cmd_syndesc *as, struct idlist *lids, struct namelis
 
 
 int
-AddToGroup(struct cmd_syndesc *as, char *arock)
+AddToGroup(struct cmd_syndesc *as, void *arock)
 {
     register afs_int32 code;
     struct cmd_item *u, *g;
@@ -501,7 +469,7 @@ AddToGroup(struct cmd_syndesc *as, char *arock)
 	for (g = as->parms[1].items; g; g = g->next) {
 	    code = pr_AddToGroup(u->data, g->data);
 	    if (code) {
-		com_err(whoami, code,
+		afs_com_err(whoami, code,
 			"; unable to add user %s to group %s %s", u->data,
 			g->data, (force ? "(ignored)" : ""));
 		if (!force)
@@ -513,7 +481,7 @@ AddToGroup(struct cmd_syndesc *as, char *arock)
 }
 
 int
-RemoveFromGroup(struct cmd_syndesc *as, char *arock)
+RemoveFromGroup(struct cmd_syndesc *as, void *arock)
 {
     register afs_int32 code;
     struct cmd_item *u, *g;
@@ -522,7 +490,7 @@ RemoveFromGroup(struct cmd_syndesc *as, char *arock)
 	for (g = as->parms[1].items; g; g = g->next) {
 	    code = pr_RemoveUserFromGroup(u->data, g->data);
 	    if (code) {
-		com_err(whoami, code,
+		afs_com_err(whoami, code,
 			"; unable to remove user %s from group %s %s",
 			u->data, g->data, (force ? "(ignored)" : ""));
 		if (!force)
@@ -534,7 +502,7 @@ RemoveFromGroup(struct cmd_syndesc *as, char *arock)
 }
 
 int
-ListMembership(struct cmd_syndesc *as, char *arock)
+ListMembership(struct cmd_syndesc *as, void *arock)
 {
     register afs_int32 code;
     idlist ids;
@@ -557,7 +525,7 @@ ListMembership(struct cmd_syndesc *as, char *arock)
 	list.namelist_len = 0;
 	code = pr_IDListMembers(ids.idlist_val[i], &list);
 	if (code) {
-	    com_err(whoami, code, "; unable to get membership of %s (id: %d)",
+	    afs_com_err(whoami, code, "; unable to get membership of %s (id: %d)",
 		    name, id);
 	    continue;
 	}
@@ -579,7 +547,7 @@ ListMembership(struct cmd_syndesc *as, char *arock)
 }
 
 int
-Delete(struct cmd_syndesc *as, char *arock)
+Delete(struct cmd_syndesc *as, void *arock)
 {
     register afs_int32 code;
     idlist ids;
@@ -598,7 +566,7 @@ Delete(struct cmd_syndesc *as, char *arock)
 
 	code = pr_DeleteByID(id);
 	if (code) {
-	    com_err(whoami, code, "deleting %s (id: %d) %s", name, id,
+	    afs_com_err(whoami, code, "deleting %s (id: %d) %s", name, id,
 		    (force ? "(ignored)" : ""));
 	    if (!force)
 		return code;
@@ -618,7 +586,7 @@ char *flags_dncase = "s mar";	/* legal member acces values */
 int flags_shift[5] = { 2, 1, 2, 2, 1 };	/* bits for each */
 
 int
-CheckEntry(struct cmd_syndesc *as, char *arock)
+CheckEntry(struct cmd_syndesc *as, void *arock)
 {
     register afs_int32 code;
     afs_int32 rcode = 1;
@@ -646,7 +614,7 @@ CheckEntry(struct cmd_syndesc *as, char *arock)
 	code = pr_ListEntry(id, &aentry);
 	if (code) {
 	    rcode = code;
-	    com_err(whoami, code, "; unable to find entry for (id: %d)", id);
+	    afs_com_err(whoami, code, "; unable to find entry for (id: %d)", id);
 	    continue;
 	}
 
@@ -655,7 +623,7 @@ CheckEntry(struct cmd_syndesc *as, char *arock)
 	code = pr_IdToName(&lids, &lnames);
 	if (code) {
 	    rcode = code;
-	    com_err(whoami, code,
+	    afs_com_err(whoami, code,
 		    "translating owner (%d) and creator (%d) ids",
 		    aentry.owner, aentry.creator);
 	    continue;
@@ -717,7 +685,7 @@ CheckEntry(struct cmd_syndesc *as, char *arock)
 }
 
 int
-ListEntries(struct cmd_syndesc *as, char *arock)
+ListEntries(struct cmd_syndesc *as, void *arock)
 {
     afs_int32 code = 0;
     afs_int32 flag, startindex, nentries, nextstartindex;
@@ -736,7 +704,7 @@ ListEntries(struct cmd_syndesc *as, char *arock)
 	    pr_ListEntries(flag, startindex, &nentries, &entriesp,
 			   &nextstartindex);
 	if (code) {
-	    com_err(whoami, code, "; unable to list entries\n");
+	    afs_com_err(whoami, code, "; unable to list entries\n");
 	    if (entriesp)
 		free(entriesp);
 	    break;
@@ -754,7 +722,7 @@ ListEntries(struct cmd_syndesc *as, char *arock)
 }
 
 int
-ChownGroup(struct cmd_syndesc *as, char *arock)
+ChownGroup(struct cmd_syndesc *as, void *arock)
 {
     register afs_int32 code;
     char *name;
@@ -764,13 +732,13 @@ ChownGroup(struct cmd_syndesc *as, char *arock)
     owner = as->parms[1].items->data;
     code = pr_ChangeEntry(name, "", 0, owner);
     if (code)
-	com_err(whoami, code, "; unable to change owner of %s to %s", name,
+	afs_com_err(whoami, code, "; unable to change owner of %s to %s", name,
 		owner);
     return code;
 }
 
 int
-ChangeName(struct cmd_syndesc *as, char *arock)
+ChangeName(struct cmd_syndesc *as, void *arock)
 {
     register afs_int32 code;
     char *oldname;
@@ -780,24 +748,24 @@ ChangeName(struct cmd_syndesc *as, char *arock)
     newname = as->parms[1].items->data;
     code = pr_ChangeEntry(oldname, newname, 0, "");
     if (code)
-	com_err(whoami, code, "; unable to change name of %s to %s", oldname,
+	afs_com_err(whoami, code, "; unable to change name of %s to %s", oldname,
 		newname);
     return code;
 }
 
 int
-ListMax(struct cmd_syndesc *as, char *arock)
+ListMax(struct cmd_syndesc *as, void *arock)
 {
     register afs_int32 code;
     afs_int32 maxUser, maxGroup;
 
     code = pr_ListMaxUserId(&maxUser);
     if (code)
-	com_err(whoami, code, "getting maximum user id");
+	afs_com_err(whoami, code, "getting maximum user id");
     else {
 	code = pr_ListMaxGroupId(&maxGroup);
 	if (code)
-	    com_err(whoami, code, "getting maximum group id");
+	    afs_com_err(whoami, code, "getting maximum group id");
 	else {
 	    printf("Max user id is %d and max group id is %d.\n", maxUser,
 		   maxGroup);
@@ -807,7 +775,7 @@ ListMax(struct cmd_syndesc *as, char *arock)
 }
 
 int
-SetMax(struct cmd_syndesc *as, char *arock)
+SetMax(struct cmd_syndesc *as, void *arock)
 {
     register afs_int32 code;
     afs_int32 maxid;
@@ -817,12 +785,12 @@ SetMax(struct cmd_syndesc *as, char *arock)
 	/* set user max */
 	code = util_GetInt32(as->parms[1].items->data, &maxid);
 	if (code) {
-	    com_err(whoami, code, "because id was: '%s'",
+	    afs_com_err(whoami, code, "because id was: '%s'",
 		    as->parms[1].items->data);
 	} else {
 	    code = pr_SetMaxUserId(maxid);
 	    if (code)
-		com_err(whoami, code, "so couldn't set Max User Id to %d",
+		afs_com_err(whoami, code, "so couldn't set Max User Id to %d",
 			maxid);
 	}
     }
@@ -830,12 +798,12 @@ SetMax(struct cmd_syndesc *as, char *arock)
 	/* set group max */
 	code = util_GetInt32(as->parms[0].items->data, &maxid);
 	if (code) {
-	    com_err(whoami, code, "because id was: '%s'",
+	    afs_com_err(whoami, code, "because id was: '%s'",
 		    as->parms[0].items->data);
 	} else {
 	    code = pr_SetMaxGroupId(maxid);
 	    if (code)
-		com_err(whoami, code, "so couldn't set Max Group Id to %d",
+		afs_com_err(whoami, code, "so couldn't set Max Group Id to %d",
 			maxid);
 	}
     }
@@ -847,7 +815,7 @@ SetMax(struct cmd_syndesc *as, char *arock)
 }
 
 int
-SetFields(struct cmd_syndesc *as, char *arock)
+SetFields(struct cmd_syndesc *as, void *arock)
 {
     register afs_int32 code;
     idlist ids;
@@ -904,7 +872,7 @@ SetFields(struct cmd_syndesc *as, char *arock)
     if (as->parms[2].items) {	/* limitgroups */
 	code = util_GetInt32(as->parms[2].items->data, &ngroups);
 	if (code) {
-	    com_err(whoami, code, "because ngroups was: '%s'",
+	    afs_com_err(whoami, code, "because ngroups was: '%s'",
 		    as->parms[2].items->data);
 	    return code;
 	}
@@ -914,7 +882,7 @@ SetFields(struct cmd_syndesc *as, char *arock)
     if (as->parms[3].items) {	/* limitgroups */
 	code = util_GetInt32(as->parms[3].items->data, &nusers);
 	if (code) {
-	    com_err(whoami, code, "because nusers was: '%s'",
+	    afs_com_err(whoami, code, "because nusers was: '%s'",
 		    as->parms[3].items->data);
 	    return code;
 	}
@@ -929,7 +897,7 @@ SetFields(struct cmd_syndesc *as, char *arock)
 	    continue;
 	code = pr_SetFieldsEntry(id, mask, flags, ngroups, nusers);
 	if (code) {
-	    com_err(whoami, code, "; unable to set fields for %s (id: %d)",
+	    afs_com_err(whoami, code, "; unable to set fields for %s (id: %d)",
 		    name, id);
 	    return code;
 	}
@@ -942,7 +910,7 @@ SetFields(struct cmd_syndesc *as, char *arock)
 }
 
 int
-ListOwned(struct cmd_syndesc *as, char *arock)
+ListOwned(struct cmd_syndesc *as, void *arock)
 {
     register afs_int32 code;
     idlist ids;
@@ -971,7 +939,7 @@ ListOwned(struct cmd_syndesc *as, char *arock)
 	    list.namelist_len = 0;
 	    code = pr_ListOwned(oid, &list, &more);
 	    if (code) {
-		com_err(whoami, code,
+		afs_com_err(whoami, code,
 			"; unable to get owner list for %s (id: %d)", name,
 			oid);
 		break;
@@ -1022,13 +990,13 @@ main(int argc, char **argv)
 {
     register afs_int32 code;
     register struct cmd_syndesc *ts;
-#if defined(SUPERGROUPS)
+
     char line[2048];
     char *cp, *lastp;
     int parsec;
     char *parsev[CMD_MAXPARMS];
     char *savec;
-#endif
+
 #ifdef WIN32
     WSADATA WSAjunk;
 #endif
@@ -1052,7 +1020,7 @@ main(int argc, char **argv)
     sigaction(SIGSEGV, &nsa, NULL);
 #endif
 
-    ts = cmd_CreateSyntax("creategroup", CreateGroup, 0,
+    ts = cmd_CreateSyntax("creategroup", CreateGroup, NULL,
 			  "create a new group");
     cmd_AddParm(ts, "-name", CMD_LIST, 0, "group name");
     cmd_AddParm(ts, "-owner", CMD_SINGLE, CMD_OPTIONAL, "owner of the group");
@@ -1061,60 +1029,60 @@ main(int argc, char **argv)
     add_std_args(ts);
     cmd_CreateAlias(ts, "cg");
 
-    ts = cmd_CreateSyntax("createuser", CreateUser, 0, "create a new user");
+    ts = cmd_CreateSyntax("createuser", CreateUser, NULL, "create a new user");
     cmd_AddParm(ts, "-name", CMD_LIST, 0, "user name");
     cmd_AddParm(ts, "-id", CMD_LIST, CMD_OPTIONAL, "user id");
     add_std_args(ts);
     cmd_CreateAlias(ts, "cu");
 
-    ts = cmd_CreateSyntax("adduser", AddToGroup, 0, "add a user to a group");
+    ts = cmd_CreateSyntax("adduser", AddToGroup, NULL, "add a user to a group");
     cmd_AddParm(ts, "-user", CMD_LIST, 0, "user name");
     cmd_AddParm(ts, "-group", CMD_LIST, 0, "group name");
     add_std_args(ts);
 
-    ts = cmd_CreateSyntax("removeuser", RemoveFromGroup, 0,
+    ts = cmd_CreateSyntax("removeuser", RemoveFromGroup, NULL,
 			  "remove a user from a group");
     cmd_AddParm(ts, "-user", CMD_LIST, 0, "user name");
     cmd_AddParm(ts, "-group", CMD_LIST, 0, "group name");
     add_std_args(ts);
 
-    ts = cmd_CreateSyntax("membership", ListMembership, 0,
+    ts = cmd_CreateSyntax("membership", ListMembership, NULL,
 			  "list membership of a user or group");
     cmd_AddParm(ts, "-nameorid", CMD_LIST, 0, "user or group name or id");
     add_std_args(ts);
     cmd_CreateAlias(ts, "groups");
 
-    ts = cmd_CreateSyntax("delete", Delete, 0,
+    ts = cmd_CreateSyntax("delete", Delete, NULL,
 			  "delete a user or group from database");
     cmd_AddParm(ts, "-nameorid", CMD_LIST, 0, "user or group name or id");
     add_std_args(ts);
 
-    ts = cmd_CreateSyntax("examine", CheckEntry, 0, "examine an entry");
+    ts = cmd_CreateSyntax("examine", CheckEntry, NULL, "examine an entry");
     cmd_AddParm(ts, "-nameorid", CMD_LIST, 0, "user or group name or id");
     add_std_args(ts);
     cmd_CreateAlias(ts, "check");
 
-    ts = cmd_CreateSyntax("chown", ChownGroup, 0,
+    ts = cmd_CreateSyntax("chown", ChownGroup, NULL,
 			  "change ownership of a group");
     cmd_AddParm(ts, "-name", CMD_SINGLE, 0, "group name");
     cmd_AddParm(ts, "-owner", CMD_SINGLE, 0, "new owner");
     add_std_args(ts);
 
-    ts = cmd_CreateSyntax("rename", ChangeName, 0, "rename user or group");
+    ts = cmd_CreateSyntax("rename", ChangeName, NULL, "rename user or group");
     cmd_AddParm(ts, "-oldname", CMD_SINGLE, 0, "old name");
     cmd_AddParm(ts, "-newname", CMD_SINGLE, 0, "new name");
     add_std_args(ts);
     cmd_CreateAlias(ts, "chname");
 
-    ts = cmd_CreateSyntax("listmax", ListMax, 0, "list max id");
+    ts = cmd_CreateSyntax("listmax", ListMax, NULL, "list max id");
     add_std_args(ts);
 
-    ts = cmd_CreateSyntax("setmax", SetMax, 0, "set max id");
+    ts = cmd_CreateSyntax("setmax", SetMax, NULL, "set max id");
     cmd_AddParm(ts, "-group", CMD_SINGLE, CMD_OPTIONAL, "group max");
     cmd_AddParm(ts, "-user", CMD_SINGLE, CMD_OPTIONAL, "user max");
     add_std_args(ts);
 
-    ts = cmd_CreateSyntax("setfields", SetFields, 0,
+    ts = cmd_CreateSyntax("setfields", SetFields, NULL,
 			  "set fields for an entry");
     cmd_AddParm(ts, "-nameorid", CMD_LIST, 0, "user or group name or id");
     cmd_AddParm(ts, "-access", CMD_SINGLE, CMD_OPTIONAL, "set privacy flags");
@@ -1126,46 +1094,41 @@ main(int argc, char **argv)
 #endif
     add_std_args(ts);
 
-    ts = cmd_CreateSyntax("listowned", ListOwned, 0,
+    ts = cmd_CreateSyntax("listowned", ListOwned, NULL,
 			  "list groups owned by an entry or zero id gets orphaned groups");
     cmd_AddParm(ts, "-nameorid", CMD_LIST, 0, "user or group name or id");
     add_std_args(ts);
 
-    ts = cmd_CreateSyntax("listentries", ListEntries, 0,
+    ts = cmd_CreateSyntax("listentries", ListEntries, NULL,
 			  "list users/groups in the protection database");
     cmd_AddParm(ts, "-users", CMD_FLAG, CMD_OPTIONAL, "list user entries");
     cmd_AddParm(ts, "-groups", CMD_FLAG, CMD_OPTIONAL, "list group entries");
     add_std_args(ts);
 
-#if defined(SUPERGROUPS)
-
-    ts = cmd_CreateSyntax("interactive", Interactive, 0,
+    ts = cmd_CreateSyntax("interactive", pts_Interactive, NULL,
 			  "enter interactive mode");
     add_std_args(ts);
     cmd_CreateAlias(ts, "in");
 
-    ts = cmd_CreateSyntax("quit", Quit, 0, "exit program");
+    ts = cmd_CreateSyntax("quit", pts_Quit, NULL, "exit program");
     add_std_args(ts);
 
-    ts = cmd_CreateSyntax("source", Source, 0, "read commands from file");
+    ts = cmd_CreateSyntax("source", pts_Source, NULL, "read commands from file");
     cmd_AddParm(ts, "-file", CMD_SINGLE, 0, "filename");
     add_std_args(ts);
 
-    ts = cmd_CreateSyntax("sleep", Sleep, 0, "pause for a bit");
+    ts = cmd_CreateSyntax("sleep", pts_Sleep, NULL, "pause for a bit");
     cmd_AddParm(ts, "-delay", CMD_SINGLE, 0, "seconds");
     add_std_args(ts);
 
-#endif /* SUPERGROUPS */
-
     cmd_SetBeforeProc(GetGlobals, 0);
 
-#if defined(SUPERGROUPS)
     finished = 1;
+    source = stdin;
     if (code = cmd_Dispatch(argc, argv)) {
 	CleanUp(NULL, NULL);
 	exit(1);
     }
-    source = stdin;
     while (!finished) {
 	if (isatty(fileno(source)))
 	    fprintf(stderr, "pts> ");
@@ -1188,7 +1151,7 @@ main(int argc, char **argv)
 	    cmd_ParseLine(line, parsev, &parsec,
 			  sizeof(parsev) / sizeof(*parsev));
 	if (code) {
-	    com_err(whoami, code, "parsing line: <%s>", line);
+	    afs_com_err(whoami, code, "parsing line: <%s>", line);
 	    exit(2);
 	}
 	savec = parsev[0];
@@ -1199,11 +1162,4 @@ main(int argc, char **argv)
     }
     CleanUp(NULL, NULL);
     exit(0);
-
-#else /* SUPERGROUPS */
-
-    cmd_SetAfterProc(CleanUp, 0);
-    code = cmd_Dispatch(argc, argv);
-    exit(code != 0);
-#endif /* SUPERGROUPS */
 }
