@@ -11,23 +11,18 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/ptserver/readpwd.c,v 1.8 2004/04/18 06:13:50 kolya Exp $");
+    ("$Header: /cvs/openafs/src/ptserver/readpwd.c,v 1.9.14.2 2007/10/30 15:16:43 shadow Exp $");
 
 #include <stdio.h>
 #ifdef AFS_NT40_ENV
 #include <WINNT/afsevent.h>
 #endif
-#ifdef HAVE_STRING_H
 #include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
 #include <rx/rx.h>
 #include <rx/xdr.h>
 #include <afs/cellconfig.h>
 #include <afs/afsutil.h>
+#include <afs/com_err.h>
 #include "ptclient.h"
 
 int
@@ -42,9 +37,8 @@ osi_audit()
 
 #include "AFS_component_version_number.c"
 
-main(argc, argv)
-     afs_int32 argc;
-     char **argv;
+int
+main(afs_int32 argc, char **argv)
 {
 
     register afs_int32 code;
