@@ -11,13 +11,12 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/ubik/recovery.c,v 1.13.2.7 2008/10/18 15:24:57 jaltman Exp $");
+    ("$Header: /cvs/openafs/src/ubik/recovery.c,v 1.14.4.1 2007/10/30 15:16:47 shadow Exp $");
 
 #include <sys/types.h>
 #ifdef AFS_NT40_ENV
 #include <winsock2.h>
 #include <time.h>
-#include <fcntl.h>
 #else
 #include <sys/file.h>
 #include <netinet/in.h>
@@ -205,7 +204,7 @@ ReplayLog(register struct ubik_dbase *adbase)
     /* for now, assume that all ops in log pertain to one transaction; see if there's a commit */
     while (1) {
 	code =
-	    (*adbase->read) (adbase, LOGFILE, &opcode, tpos,
+	    (*adbase->read) (adbase, LOGFILE, (char *)&opcode, tpos,
 			     sizeof(afs_int32));
 	if (code != sizeof(afs_int32))
 	    break;
@@ -220,7 +219,7 @@ ReplayLog(register struct ubik_dbase *adbase)
 	} else if (opcode == LOGTRUNCATE) {
 	    tpos += 4;
 	    code =
-		(*adbase->read) (adbase, LOGFILE, buffer, tpos,
+		(*adbase->read) (adbase, LOGFILE, (char *)buffer, tpos,
 				 2 * sizeof(afs_int32));
 	    if (code != 2 * sizeof(afs_int32))
 		break;		/* premature eof or io error */
@@ -228,7 +227,7 @@ ReplayLog(register struct ubik_dbase *adbase)
 	} else if (opcode == LOGDATA) {
 	    tpos += 4;
 	    code =
-		(*adbase->read) (adbase, LOGFILE, buffer, tpos,
+		(*adbase->read) (adbase, LOGFILE, (char *)buffer, tpos,
 				 3 * sizeof(afs_int32));
 	    if (code != 3 * sizeof(afs_int32))
 		break;
@@ -248,7 +247,7 @@ ReplayLog(register struct ubik_dbase *adbase)
 	syncFile = -1;
 	while (1) {
 	    code =
-		(*adbase->read) (adbase, LOGFILE, &opcode, tpos,
+		(*adbase->read) (adbase, LOGFILE, (char *)&opcode, tpos,
 				 sizeof(afs_int32));
 	    if (code != sizeof(afs_int32))
 		break;
@@ -260,11 +259,11 @@ ReplayLog(register struct ubik_dbase *adbase)
 	    else if (opcode == LOGEND) {
 		tpos += 4;
 		code =
-		    (*adbase->read) (adbase, LOGFILE, buffer, tpos,
+		    (*adbase->read) (adbase, LOGFILE, (char *)buffer, tpos,
 				     2 * sizeof(afs_int32));
 		if (code != 2 * sizeof(afs_int32))
 		    return UBADLOG;
-		code = (*adbase->setlabel) (adbase, 0, buffer);
+		code = (*adbase->setlabel) (adbase, 0, (ubik_version *)buffer);
 		if (code)
 		    return code;
 		logIsGood = 1;
@@ -272,7 +271,7 @@ ReplayLog(register struct ubik_dbase *adbase)
 	    } else if (opcode == LOGTRUNCATE) {
 		tpos += 4;
 		code =
-		    (*adbase->read) (adbase, LOGFILE, buffer, tpos,
+		    (*adbase->read) (adbase, LOGFILE, (char *)buffer, tpos,
 				     2 * sizeof(afs_int32));
 		if (code != 2 * sizeof(afs_int32))
 		    break;	/* premature eof or io error */
@@ -285,7 +284,7 @@ ReplayLog(register struct ubik_dbase *adbase)
 	    } else if (opcode == LOGDATA) {
 		tpos += 4;
 		code =
-		    (*adbase->read) (adbase, LOGFILE, buffer, tpos,
+		    (*adbase->read) (adbase, LOGFILE, (char *)buffer, tpos,
 				     3 * sizeof(afs_int32));
 		if (code != 3 * sizeof(afs_int32))
 		    break;
@@ -308,12 +307,12 @@ ReplayLog(register struct ubik_dbase *adbase)
 		    thisSize = (len > sizeof(data) ? sizeof(data) : len);
 		    /* copy sizeof(data) buffer bytes at a time */
 		    code =
-			(*adbase->read) (adbase, LOGFILE, data, tpos,
+			(*adbase->read) (adbase, LOGFILE, (char *)data, tpos,
 					 thisSize);
 		    if (code != thisSize)
 			return UBADLOG;
 		    code =
-			(*adbase->write) (adbase, tfile, data, filePos,
+			(*adbase->write) (adbase, tfile, (char *)data, filePos,
 					  thisSize);
 		    if (code != thisSize)
 			return UBADLOG;
@@ -412,8 +411,8 @@ urecovery_Initialize(register struct ubik_dbase *adbase)
  * requests.  However, the recovery module still has one more task:
  * propagating the dbase out to everyone who is up in the network.
  */
-void *
-urecovery_Interact(void *dummy)
+int
+urecovery_Interact(void)
 {
     afs_int32 code, tcode;
     struct ubik_server *bestServer = NULL;
@@ -426,14 +425,9 @@ urecovery_Interact(void *dummy)
     struct timeval tv;
     int length, tlen, offset, file, nbytes;
     struct rx_call *rxcall;
-    char tbuffer[1024];
+    char tbuffer[256];
     struct ubik_stat ubikstat;
     struct in_addr inAddr;
-#ifndef OLD_URECOVERY
-    char pbuffer[1028];
-    int flen, fd = -1;
-    afs_int32 epoch, pass;
-#endif
 
     /* otherwise, begin interaction */
     urecovery_state = 0;
@@ -536,7 +530,11 @@ urecovery_Interact(void *dummy)
 	    urecovery_state |= UBIK_RECHAVEDB;
 	} else {
 	    /* we don't have the best version; we should fetch it. */
+#if defined(UBIK_PAUSE)
 	    DBHOLD(ubik_dbase);
+#else
+	    ObtainWriteLock(&ubik_dbase->versionLock);
+#endif /* UBIK_PAUSE */
 	    urecovery_AbortAll(ubik_dbase);
 
 	    /* Rx code to do the Bulk fetch */
@@ -552,7 +550,7 @@ urecovery_Interact(void *dummy)
 		ubik_dprint("StartDiskGetFile failed=%d\n", code);
 		goto FetchEndCall;
 	    }
-	    nbytes = rx_Read(rxcall, &length, sizeof(afs_int32));
+	    nbytes = rx_Read(rxcall, (char *)&length, sizeof(afs_int32));
 	    length = ntohl(length);
 	    if (nbytes != sizeof(afs_int32)) {
 		ubik_dprint("Rx-read length error=%d\n", code = BULK_ERROR);
@@ -560,72 +558,40 @@ urecovery_Interact(void *dummy)
 		goto FetchEndCall;
 	    }
 
-#ifdef OLD_URECOVERY
-	    /* Truncate the file first */
+	    /* Truncate the file firest */
 	    code = (*ubik_dbase->truncate) (ubik_dbase, file, 0);
 	    if (code) {
 		ubik_dprint("truncate io error=%d\n", code);
 		goto FetchEndCall;
 	    }
-	    tversion.counter = 0;
-#endif
+
 	    /* give invalid label during file transit */
 	    tversion.epoch = 0;
+	    tversion.counter = 0;
 	    code = (*ubik_dbase->setlabel) (ubik_dbase, file, &tversion);
 	    if (code) {
 		ubik_dprint("setlabel io error=%d\n", code);
 		goto FetchEndCall;
 	    }
-#ifndef OLD_URECOVERY
-	    flen = length;
-	    afs_snprintf(pbuffer, sizeof(pbuffer), "%s.DB0.TMP", ubik_dbase->pathName);
-	    fd = open(pbuffer, O_CREAT | O_RDWR | O_TRUNC, 0600);
-	    if (fd < 0) {
-		code = errno;
-		goto FetchEndCall;
-	    }
-	    code = lseek(fd, HDRSIZE, 0);
-	    if (code != HDRSIZE) {
-		close(fd);
-		goto FetchEndCall;
-	    }
-#endif
 
-	    pass = 0;
 	    while (length > 0) {
 		tlen = (length > sizeof(tbuffer) ? sizeof(tbuffer) : length);
-#ifndef AFS_PTHREAD_ENV
-		if (pass % 4 == 0)
-		    IOMGR_Poll();
-#endif
 		nbytes = rx_Read(rxcall, tbuffer, tlen);
 		if (nbytes != tlen) {
 		    ubik_dprint("Rx-read bulk error=%d\n", code = BULK_ERROR);
 		    code = EIO;
-		    close(fd);
 		    goto FetchEndCall;
 		}
-#ifdef OLD_URECOVERY
 		nbytes =
 		    (*ubik_dbase->write) (ubik_dbase, file, tbuffer, offset,
 					  tlen);
-#else
-		nbytes = write(fd, tbuffer, tlen);
-		pass++;
-#endif
 		if (nbytes != tlen) {
 		    code = UIOERROR;
-		    close(fd);
 		    goto FetchEndCall;
 		}
 		offset += tlen;
 		length -= tlen;
 	    }
-#ifndef OLD_URECOVERY
-	    code = close(fd);
-	    if (code)
-		goto FetchEndCall;
-#endif	    
 	    code = EndDISK_GetFile(rxcall, &tversion);
 	  FetchEndCall:
 	    tcode = rx_EndCall(rxcall, code);
@@ -636,53 +602,27 @@ urecovery_Interact(void *dummy)
 		urecovery_state |= UBIK_RECHAVEDB;
 		memcpy(&ubik_dbase->version, &tversion,
 		       sizeof(struct ubik_version));
-#ifdef OLD_URECOVERY
 		(*ubik_dbase->sync) (ubik_dbase, 0);	/* get data out first */
-#else
-		afs_snprintf(tbuffer, sizeof(tbuffer), "%s.DB0", ubik_dbase->pathName);
-#ifdef AFS_NT40_ENV
-		afs_snprintf(pbuffer, sizeof(pbuffer), "%s.DB0.OLD", ubik_dbase->pathName);
-		code = unlink(pbuffer);
-		if (!code)
-		    code = rename(tbuffer, pbuffer);
-		afs_snprintf(pbuffer, sizeof(pbuffer), "%s.DB0.TMP", ubik_dbase->pathName);
-#endif
-		if (!code) 
-		    code = rename(pbuffer, tbuffer);
-		if (!code) 
-		    code = (*ubik_dbase->open) (ubik_dbase, 0);
-		if (!code)
-#endif
 		/* after data is good, sync disk with correct label */
 		code =
 		    (*ubik_dbase->setlabel) (ubik_dbase, 0,
 					     &ubik_dbase->version);
-#ifndef OLD_URECOVERY
-#ifdef AFS_NT40_ENV
-		afs_snprintf(pbuffer, sizeof(pbuffer), "%s.DB0.OLD", ubik_dbase->pathName);
-		unlink(pbuffer);
-#endif
-#endif
 	    }
 	    if (code) {
-#ifndef OLD_URECOVERY
-		unlink(pbuffer);
-		/* 
-		 * We will effectively invalidate the old data forever now.
-		 * Unclear if we *should* but we do.
-		 */
-#endif
 		ubik_dbase->version.epoch = 0;
 		ubik_dbase->version.counter = 0;
 		ubik_print("Ubik: Synchronize database failed (error = %d)\n",
 			   code);
 	    } else {
 		ubik_print("Ubik: Synchronize database completed\n");
-		urecovery_state |= UBIK_RECHAVEDB;
 	    }
 	    udisk_Invalidate(ubik_dbase, 0);	/* data has changed */
 	    LWP_NoYieldSignal(&ubik_dbase->version);
+#if defined(UBIK_PAUSE)
 	    DBRELE(ubik_dbase);
+#else
+	    ReleaseWriteLock(&ubik_dbase->versionLock);
+#endif /* UBIK_PAUSE */
 	}
 #if defined(UBIK_PAUSE)
 	if (!(urecovery_state & UBIK_RECSYNCSITE))
@@ -697,7 +637,11 @@ urecovery_Interact(void *dummy)
 	 * database and overwrite this one.
 	 */
 	if (ubik_dbase->version.epoch == 1) {
+#if defined(UBIK_PAUSE)
 	    DBHOLD(ubik_dbase);
+#else
+	    ObtainWriteLock(&ubik_dbase->versionLock);
+#endif /* UBIK_PAUSE */
 	    urecovery_AbortAll(ubik_dbase);
 	    ubik_epochTime = 2;
 	    ubik_dbase->version.epoch = ubik_epochTime;
@@ -706,7 +650,11 @@ urecovery_Interact(void *dummy)
 		(*ubik_dbase->setlabel) (ubik_dbase, 0, &ubik_dbase->version);
 	    udisk_Invalidate(ubik_dbase, 0);	/* data may have changed */
 	    LWP_NoYieldSignal(&ubik_dbase->version);
+#if defined(UBIK_PAUSE)
 	    DBRELE(ubik_dbase);
+#else
+	    ReleaseWriteLock(&ubik_dbase->versionLock);
+#endif /* UBIK_PAUSE */
 	}
 
 	/* Check the other sites and send the database to them if they
@@ -716,7 +664,11 @@ urecovery_Interact(void *dummy)
 	    /* now propagate out new version to everyone else */
 	    dbok = 1;		/* start off assuming they all worked */
 
+#if defined(UBIK_PAUSE)
 	    DBHOLD(ubik_dbase);
+#else
+	    ObtainWriteLock(&ubik_dbase->versionLock);
+#endif /* UBIK_PAUSE */
 	    /*
 	     * Check if a write transaction is in progress. We can't send the
 	     * db when a write is in progress here because the db would be
@@ -732,12 +684,20 @@ urecovery_Interact(void *dummy)
 		tv.tv_sec = 0;
 		tv.tv_usec = 50000;
 		while ((ubik_dbase->flags & DBWRITING) && (safety < 500)) {
+#if defined(UBIK_PAUSE)
 		    DBRELE(ubik_dbase);
+#else
+		    ReleaseWriteLock(&ubik_dbase->versionLock);
+#endif /* UBIK_PAUSE */
 		    /* sleep for a little while */
 		    IOMGR_Select(0, 0, 0, 0, &tv);
 		    tv.tv_usec += 10000;
 		    safety++;
+#if defined(UBIK_PAUSE)
 		    DBHOLD(ubik_dbase);
+#else
+		    ObtainWriteLock(&ubik_dbase->versionLock);
+#endif /* UBIK_PAUSE */
 		}
 	    }
 
@@ -804,12 +764,15 @@ urecovery_Interact(void *dummy)
 		    ts->currentDB = 1;
 		}
 	    }
+#if defined(UBIK_PAUSE)
 	    DBRELE(ubik_dbase);
+#else
+	    ReleaseWriteLock(&ubik_dbase->versionLock);
+#endif /* UBIK_PAUSE */
 	    if (dbok)
 		urecovery_state |= UBIK_RECSENTDB;
 	}
     }
-    return NULL;
 }
 
 /*
