@@ -4,28 +4,22 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/util/snprintf.c,v 1.20 2003/11/27 01:23:13 shadow Exp $");
+    ("$Header: /cvs/openafs/src/util/snprintf.c,v 1.24.4.1 2007/10/30 15:16:48 shadow Exp $");
 
 #include <sys/types.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <ctype.h>
+#include <string.h>
 #ifndef AFS_NT40_ENV
 #include <netinet/in.h>
 #include <netdb.h>
 #else
 #include <winsock2.h>
 #endif
-#if defined(AFS_AIX32_ENV) || defined(AFS_SUN_ENV) || defined(AFS_XBSD_ENV) || defined(AFS_HPUX_ENV)
+#if defined(AFS_AIX32_ENV) || defined(AFS_SUN_ENV) || defined(AFS_XBSD_ENV) || defined(AFS_HPUX_ENV) || defined(AFS_SGI65_ENV)
 #include <sys/socket.h>
 #endif
-#ifdef HAVE_STRING_H
-#include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
 
 #define MAXPREC 100
 
@@ -43,7 +37,7 @@ mkint(char *buf, afs_uintmax_t val, int base, int uc, unsigned prec)
     int len = 0, dig, i;
 
     while (val) {
-	dig = val % base;
+	dig = (int) (val % base);
 	val = (val - dig) / base;
 	if (dig < 10)
 	    dig = dig + '0';
@@ -137,7 +131,8 @@ mkint(char *buf, afs_uintmax_t val, int base, int uc, unsigned prec)
 int
 afs_vsnprintf(char *p, size_t avail, const char *fmt, va_list ap)
 {
-    unsigned int width, precision, haveprec, len;
+    unsigned int width, precision, haveprec;
+    size_t len;
     int ljust, plsign, spsign, altform, zfill;
     int hflag, lflag, count, *countp, j;
     char *x, *y, xbuf[MAXPREC + 21], fbuf[20];
@@ -408,7 +403,7 @@ afs_vsnprintf(char *p, size_t avail, const char *fmt, va_list ap)
 				 * if spsign specified, IPaddr fields are space-filled to 3 digits
 				 */
 	    UVAL = va_arg(ap, unsigned long);
-	    ia.s_addr = UVAL;
+	    ia.s_addr = (unsigned long)UVAL;
 	    if (haveprec && !precision)
 		he = 0;
 	    else
@@ -428,7 +423,7 @@ afs_vsnprintf(char *p, size_t avail, const char *fmt, va_list ap)
 			    *y = toupper(*y);
 		}
 	    } else {
-		UVAL = ntohl(UVAL);
+		UVAL = ntohl((unsigned long)UVAL);
 		if (zfill) {
 		    x = "%03u.%03u.%03u.%03u";
 		} else if (spsign) {
@@ -436,9 +431,11 @@ afs_vsnprintf(char *p, size_t avail, const char *fmt, va_list ap)
 		} else {
 		    x = "%u.%u.%u.%u";
 		}
-		sprintf(xbuf, x, (UVAL & 0xff000000) >> 24,
-			(UVAL & 0x00ff0000) >> 16, (UVAL & 0x0000ff00) >> 8,
-			(UVAL & 0x000000ff));
+		/* typecast to whatever '%u' is! */
+		sprintf(xbuf, x, (unsigned int)((UVAL & 0xff000000) >> 24),
+			(unsigned int)((UVAL & 0x00ff0000) >> 16), 
+			(unsigned int)((UVAL & 0x0000ff00) >> 8),
+			(unsigned int)(UVAL & 0x000000ff));
 		x = xbuf;
 		len = strlen(xbuf);
 	    }
@@ -508,7 +505,7 @@ afs_snprintf(char *p, size_t avail, const char *fmt, ...)
 
 #if defined(AFS_OSF20_ENV) && !defined(AFS_DUX50_ENV) || defined(AFS_AIX32_ENV) || (defined(AFS_SUN55_ENV) && !defined(AFS_SUN56_ENV)) || !defined(HAVE_VSNPRINTF)
 
-#ifdef AFS_AIX51_ENV
+#if defined(AFS_AIX51_ENV) || defined(AFS_NT40_ENV)
 int
 vsnprintf(char *p, size_t avail, const char *fmt, va_list ap)
 #else
@@ -518,7 +515,7 @@ vsnprintf(char *p, unsigned int avail, char *fmt, va_list ap)
 {
     int result;
     result = afs_vsnprintf(p, avail, fmt, ap);
-#ifdef AFS_AIX51_ENV
+#if defined(AFS_AIX51_ENV) || defined(AFS_NT40_ENV)
     return result;
 #endif
 }
