@@ -14,7 +14,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/venus/fstrace.c,v 1.23 2006/04/04 20:51:19 shadow Exp $");
+    ("$Header: /cvs/openafs/src/venus/fstrace.c,v 1.23.2.2 2007/10/31 04:09:41 shadow Exp $");
 
 #include <stdio.h>
 #include <sys/types.h>
@@ -27,13 +27,7 @@ RCSID
 #undef abs
 #include <stdlib.h>
 
-#ifdef HAVE_STRING_H
 #include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
 
 #include <afs/stds.h>
 #include <afs/cmd.h>
@@ -2713,10 +2707,8 @@ GetBulkLogInfo()
 }
 
 
-static
-DoDump(as, arock)
-     register struct cmd_syndesc *as;
-     char *arock;
+static int
+DoDump(struct cmd_syndesc *as, void *arock)
 {
     afs_int32 code = 0;
     afs_int32 tcode;
@@ -2792,7 +2784,7 @@ SetUpDump()
     struct cmd_syndesc *dumpSyntax;
 
     dumpSyntax =
-	cmd_CreateSyntax("dump", DoDump, (char *)NULL, "dump AFS trace logs");
+	cmd_CreateSyntax("dump", DoDump, NULL, "dump AFS trace logs");
     (void)cmd_AddParm(dumpSyntax, "-set", CMD_LIST, CMD_OPTIONAL, "set_name");
     (void)cmd_AddParm(dumpSyntax, "-follow", CMD_SINGLE, CMD_OPTIONAL,
 		      "log_name");
@@ -2802,10 +2794,8 @@ SetUpDump()
 		      "seconds_between_reads");
 }
 
-static
-DoShowLog(as, arock)
-     register struct cmd_syndesc *as;
-     char *arock;
+static int
+DoShowLog(register struct cmd_syndesc *as, void *arock)
 {
     afs_int32 retVal = 0;
     afs_int32 code = 0;
@@ -2868,17 +2858,15 @@ SetUpShowLog()
     struct cmd_syndesc *showSyntax;
 
     showSyntax =
-	cmd_CreateSyntax("lslog", DoShowLog, (char *)NULL,
+	cmd_CreateSyntax("lslog", DoShowLog, NULL,
 			 "list available logs");
     (void)cmd_AddParm(showSyntax, "-set", CMD_LIST, CMD_OPTIONAL, "set_name");
     (void)cmd_AddParm(showSyntax, "-log", CMD_LIST, CMD_OPTIONAL, "log_name");
     (void)cmd_AddParm(showSyntax, "-long", CMD_FLAG, CMD_OPTIONAL, "");
 }
 
-static
-DoShowSet(as, arock)
-     register struct cmd_syndesc *as;
-     char *arock;
+static int
+DoShowSet(register struct cmd_syndesc *as, void *arock)
 {
     afs_int32 retVal = 0;
     afs_int32 code = 0;
@@ -2926,15 +2914,13 @@ SetUpShowSet()
     struct cmd_syndesc *showSyntax;
 
     showSyntax =
-	cmd_CreateSyntax("lsset", DoShowSet, (char *)NULL,
+	cmd_CreateSyntax("lsset", DoShowSet, NULL,
 			 "list available event sets");
     (void)cmd_AddParm(showSyntax, "-set", CMD_LIST, CMD_OPTIONAL, "set_name");
 }
 
-static
-DoClear(as, arock)
-     register struct cmd_syndesc *as;
-     char *arock;
+static int
+DoClear(register struct cmd_syndesc *as, void *arock)
 {
     afs_int32 retVal = 0;
     afs_int32 code = 0;
@@ -2985,7 +2971,7 @@ SetUpClear()
     struct cmd_syndesc *clearSyntax;
 
     clearSyntax =
-	cmd_CreateSyntax("clear", DoClear, (char *)NULL,
+	cmd_CreateSyntax("clear", DoClear, NULL,
 			 "clear logs by logname or by event set");
     (void)cmd_AddParm(clearSyntax, "-set", CMD_LIST, CMD_OPTIONAL,
 		      "set_name");
@@ -2993,10 +2979,8 @@ SetUpClear()
 		      "log_name");
 }
 
-static
-DoSet(as, arock)
-     register struct cmd_syndesc *as;
-     char *arock;
+static int
+DoSet(register struct cmd_syndesc *as, void *arock)
 {
     afs_int32 retVal = 0;
     afs_int32 code = 0;
@@ -3074,7 +3058,7 @@ SetUpSet()
     struct cmd_syndesc *setSyntax;
 
     setSyntax =
-	cmd_CreateSyntax("setset", DoSet, (char *)NULL,
+	cmd_CreateSyntax("setset", DoSet, NULL,
 			 "set state of event sets");
     (void)cmd_AddParm(setSyntax, "-set", CMD_LIST, CMD_OPTIONAL, "set_name");
     (void)cmd_AddParm(setSyntax, "-active", CMD_FLAG, CMD_OPTIONAL, "");
@@ -3082,10 +3066,8 @@ SetUpSet()
     (void)cmd_AddParm(setSyntax, "-dormant", CMD_FLAG, CMD_OPTIONAL, "");
 }
 
-static
-DoResize(as, arock)
-     register struct cmd_syndesc *as;
-     char *arock;
+static int
+DoResize(register struct cmd_syndesc *as, void *arock)
 {
     afs_int32 retVal = 0;
     afs_int32 code = 0;
@@ -3133,7 +3115,7 @@ SetUpResize()
     struct cmd_syndesc *setsizeSyntax;
 
     setsizeSyntax =
-	cmd_CreateSyntax("setlog", DoResize, (char *)NULL,
+	cmd_CreateSyntax("setlog", DoResize, NULL,
 			 "set the size of a log");
     (void)cmd_AddParm(setsizeSyntax, "-log", CMD_LIST, CMD_OPTIONAL,
 		      "log_name");
