@@ -11,7 +11,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/viced/fsprobe.c,v 1.12 2006/02/22 20:29:01 rees Exp $");
+    ("$Header: /cvs/openafs/src/viced/fsprobe.c,v 1.12.2.2 2007/10/30 15:16:49 shadow Exp $");
 
 #include <afs/stds.h>
 #include <afs/afsint.h>
@@ -21,15 +21,7 @@ RCSID
 #include <arpa/inet.h>
 #include <netdb.h>
 #include <ubik.h>
-
-#ifdef HAVE_STRING_H
 #include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
-
 
 struct ubik_client *cstruct;
 struct rx_connection *serverconns[MAXSERVERS];
@@ -168,7 +160,7 @@ main(int argc, char **argv)
 	    } else if (!strcmp(oper, "fsstats")) {
 		struct afsStatistics stats;
 
-		code = ubik_Call(AFS_GetStatistics, cstruct, 0, &stats);
+		code = ubik_AFS_GetStatistics(cstruct, 0, &stats);
 		printf("return code is %d\n", code);
 	    } else if (!strcmp(oper, "fd")) {
 		code = FetchData(argp);
@@ -364,7 +356,7 @@ FetchStatus(char **argp)
     fid.Vnode = vnode;
     fid.Unique = unique;
     code =
-	ubik_Call(AFS_FetchStatus, cstruct, 0, &fid, &hyp0, 0, &OutStatus,
+	ubik_AFS_FetchStatus(cstruct, 0, &fid, &hyp0, 0, &OutStatus,
 		  &Token, &tsync);
     return (code);
 }
@@ -390,7 +382,7 @@ FetchACL(char **argp)
     fid.Vnode = vnode;
     fid.Unique = unique;
     code =
-	ubik_Call(AFS_FetchACL, cstruct, 0, &fid, &hyp0, 0, &AccessList,
+	ubik_AFS_FetchACL(cstruct, 0, &fid, &hyp0, 0, &AccessList,
 		  &OutStatus, &tsync);
     return (code);
 }
@@ -515,7 +507,7 @@ StoreStatus(char **argp)
 	InStatus.mask |= AFS_SETLENGTH;
     }
     code =
-	ubik_Call(AFS_StoreStatus, cstruct, 0, &fid, &InStatus, &hyp0, 0,
+	ubik_AFS_StoreStatus(cstruct, 0, &fid, &InStatus, &hyp0, 0,
 		  &OutStatus, &tsync);
     return (code);
 }
@@ -546,7 +538,7 @@ StoreACL(char **argp)
     AccessList.afsACL_len = strlen(string) + 1;
     AccessList.afsACL_val = string;
     code =
-	ubik_Call(AFS_StoreACL, cstruct, 0, &fid, &AccessList, &hyp0, 0,
+	ubik_AFS_StoreACL(cstruct, 0, &fid, &AccessList, &hyp0, 0,
 		  &OutStatus, &tsync);
     return (code);
 }
@@ -576,7 +568,7 @@ RemoveFile(char **argp)
     memset(&nameFid, 0, sizeof(struct afsFidName));
     strcpy(nameFid.name, name);
     code =
-	ubik_Call(AFS_RemoveFile, cstruct, 0, &fid, &nameFid, &hyp0, 0,
+	ubik_AFS_RemoveFile(cstruct, 0, &fid, &nameFid, &hyp0, 0,
 		  &OutDirStatus, &OutFidStatus, &outFid, &tsync);
     return (code);
 }
@@ -624,7 +616,7 @@ CreateFile(char **argp)
 	InStatus.mask |= AFS_SETLENGTH;
     }
     code =
-	ubik_Call(AFS_CreateFile, cstruct, 0, &fid, name, &InStatus, &hyp0, 0,
+	ubik_AFS_CreateFile(cstruct, 0, &fid, name, &InStatus, &hyp0, 0,
 		  &outFid, &OutFidStatus, &OutDirStatus, &Token, &tsync);
     return (code);
 }
@@ -667,7 +659,7 @@ Rename(char **argp)
     memset(&NewName, 0, sizeof(struct afsFidName));
     strcpy(NewName.name, nname);
     code =
-	ubik_Call(AFS_Rename, cstruct, 0, &OldDirFid, &OldName, &NewDirFid,
+	ubik_AFS_Rename(cstruct, 0, &OldDirFid, &OldName, &NewDirFid,
 		  &NewName, &hyp0, 0, &OutOldDirStatus, &OutNewDirStatus,
 		  &OutOldFileFid, &OutOldFileStatus, &OutNewFileFid,
 		  &OutNewFileStatus, &tsync);
@@ -719,7 +711,7 @@ Symlink(char **argp)
 	InStatus.mask |= AFS_SETLENGTH;
     }
     code =
-	ubik_Call(AFS_Symlink, cstruct, 0, &fid, name, linkcontents,
+	ubik_AFS_Symlink(cstruct, 0, &fid, name, linkcontents,
 		  &InStatus, &hyp0, 0, &outFid, &OutFidStatus, &OutDirStatus,
 		  &Token, &tsync);
     return (code);
@@ -755,7 +747,7 @@ HardLink(char **argp)
     existingFid.Vnode = vnode;
     existingFid.Unique = unique;
     code =
-	ubik_Call(AFS_HardLink, cstruct, 0, &fid, name, &existingFid, &hyp0,
+	ubik_AFS_HardLink(cstruct, 0, &fid, name, &existingFid, &hyp0,
 		  0, &OutFidStatus, &OutDirStatus, &tsync);
     return (code);
 }
@@ -803,7 +795,7 @@ MakeDir(char **argp)
 	InStatus.mask |= AFS_SETLENGTH;
     }
     code =
-	ubik_Call(AFS_MakeDir, cstruct, 0, &fid, name, &InStatus, &hyp0, 0,
+	ubik_AFS_MakeDir(cstruct, 0, &fid, name, &InStatus, &hyp0, 0,
 		  &outFid, &OutFidStatus, &OutDirStatus, &Token, &tsync);
     return (code);
 }
@@ -833,7 +825,7 @@ RemoveDir(char **argp)
     memset(&nameFid, 0, sizeof(struct afsFidName));
     strcpy(nameFid.name, name);
     code =
-	ubik_Call(AFS_RemoveDir, cstruct, 0, &fid, &nameFid, &hyp0, 0,
+	ubik_AFS_RemoveDir(cstruct, 0, &fid, &nameFid, &hyp0, 0,
 		  &OutDirStatus, &outFid, &tsync);
     return (code);
 }
@@ -930,7 +922,7 @@ Lookup(char **argp)
     name = &argp[0][0];
     ++argp;
     code =
-	ubik_Call(AFS_Lookup, cstruct, 0, &fid, name, &hyp0, 0, &outFid,
+	ubik_AFS_Lookup(cstruct, 0, &fid, name, &hyp0, 0, &outFid,
 		  &OutFidStatus, &OutDirStatus, &tsync);
     return (code);
 }
@@ -959,7 +951,7 @@ GetToken(char **argp)
     memset(&MinToken, 0, sizeof(struct afsToken));
     MinToken.tokenID.low = tokenId;	/* XXX */
     code =
-	ubik_Call(AFS_GetToken, cstruct, 0, &fid, &MinToken, &hyp0, 0,
+	ubik_AFS_GetToken(cstruct, 0, &fid, &MinToken, &hyp0, 0,
 		  &RealToken, &OutStatus, &tsync);
     return (code);
 }
@@ -1005,7 +997,7 @@ KeepAlive(char **argp)
     fex.afsBulkFEX_val = &fx;
     fex.afsBulkFEX_len = 1;
     code =
-	ubik_Call(AFS_BulkKeepAlive, cstruct, 0, &fex, numExec, 0, 0, 0,
+	ubik_AFS_BulkKeepAlive(cstruct, 0, &fex, numExec, 0, 0, 0,
 		  &spare4);
     return (code);
 }
