@@ -22,11 +22,12 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/viced/viced.c,v 1.75.2.1 2006/06/07 04:40:43 shadow Exp $");
+    ("$Header: /cvs/openafs/src/viced/viced.c,v 1.75.2.16 2007/10/30 15:16:50 shadow Exp $");
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
+#include <string.h>
 #include <sys/types.h>
 #include <afs/procmgmt.h>	/* signal(), kill(), wait(), etc. */
 #include <sys/stat.h>
@@ -41,14 +42,6 @@ RCSID
 #include <netdb.h>
 #include <unistd.h>		/* sysconf() */
 
-#ifdef HAVE_STRING_H
-#include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
-
 #ifndef ITIMER_REAL
 #include <sys/time.h>
 #endif /* ITIMER_REAL */
@@ -77,7 +70,6 @@ RCSID
 #include <afs/acl.h>
 #include <afs/prs_fs.h>
 #include <rx/rx.h>
-#include <rx/rxkad.h>
 #include <afs/keys.h>
 #include <afs/afs_args.h>
 #include <afs/vlserver.h>
@@ -89,6 +81,7 @@ RCSID
 #ifndef AFS_NT40_ENV
 #include <afs/netutils.h>
 #endif
+#include "viced_prototypes.h"
 #include "viced.h"
 #include "host.h"
 #ifdef AFS_PTHREAD_ENV
@@ -155,8 +148,6 @@ struct afsconf_dir *confDir;	/* Configuration dir object */
 
 int restartMode = RESTART_ORDINARY;
 
-int Testing = 0;		/* for ListViceInodes */
-
 /*
  * Home for the performance statistics.
  */
@@ -172,6 +163,7 @@ int SawPctSpare;
 int debuglevel = 0;
 int printBanner = 0;
 int rxJumbograms = 1;		/* default is to send and receive jumbograms. */
+int rxBind = 0;		/* don't bind */
 int rxMaxMTU = -1;
 afs_int32 implicitAdminRights = PRSFS_LOOKUP;	/* The ADMINISTER right is 
 						 * already implied */
@@ -199,9 +191,14 @@ int busy_threshold = 600;
 int abort_threshold = 10;
 int udpBufSize = 0;		/* UDP buffer size for receive */
 int sendBufSize = 16384;	/* send buffer size */
+int saneacls = 0;		/* Sane ACLs Flag */
 
 struct timeval tp;
 
+#ifdef AFS_PTHREAD_ENV
+pthread_key_t viced_uclient_key;
+#endif
+
 /*
  * FileServer's name and IP address, both network byte order and
  * host byte order.
@@ -226,6 +223,11 @@ static void FlagMsg();
  * certain background threads before we are allowed to dump state to
  * disk
  */
+
+#if !defined(PTHREAD_RWLOCK_INITIALIZER) && defined(AFS_DARWIN80_ENV)
+#define PTHREAD_RWLOCK_INITIALIZER {0x2DA8B3B4, {0}}
+#endif
+
 struct fs_state fs_state = 
     { FS_MODE_NORMAL, 
       0, 
@@ -889,6 +891,7 @@ FlagMsg()
     strcat(buffer, "[-rxdbg (enable rx debugging)] ");
     strcat(buffer, "[-rxdbge (enable rxevent debugging)] ");
     strcat(buffer, "[-rxmaxmtu <bytes>] ");
+    strcat(buffer, "[-rxbind (bind the Rx socket to one address)] ");
 #ifdef AFS_DEMAND_ATTACH_FS
     strcat(buffer, "[-fs-state-dont-save (disable state save during shutdown)] ");
     strcat(buffer, "[-fs-state-dont-restore (disable state restore during startup)] ");
@@ -1266,6 +1269,8 @@ ParseArgs(int argc, char *argv[])
 #endif
 	else if (!strcmp(argv[i], "-nojumbo")) {
 	    rxJumbograms = 0;
+	} else if (!strcmp(argv[i], "-rxbind")) {
+	    rxBind = 1;
 	} else if (!strcmp(argv[i], "-rxmaxmtu")) {
 	    if ((i + 1) >= argc) {
 		fprintf(stderr, "missing argument for -rxmaxmtu\n"); 
@@ -1373,6 +1378,9 @@ ParseArgs(int argc, char *argv[])
 	    /* set syslog logging flag */
 	    mrafsStyleLogs = 1;
 	} 
+	else if (strcmp(argv[i], "-saneacls") == 0) {
+	    saneacls = 1;
+	}
 	else {
 	    return (-1);
 	}
@@ -1515,6 +1523,11 @@ InitPR()
 		("Couldn't initialize protection library; code=%d.\n", code));
 	return code;
     }
+
+#ifdef AFS_PTHREAD_ENV
+    assert(pthread_key_create(&viced_uclient_key, NULL) == 0);
+#endif
+
     SystemId = SYSADMINID;
     SystemAnyUser = ANYUSERID;
     SystemAnyUserCPS.prlist_len = 0;
@@ -1582,6 +1595,7 @@ vl_Initialize(const char *confDir)
 			     info.hostAddr[i].sin_port, USER_SERVICE_ID, sc,
 			     scIndex);
     code = ubik_ClientInit(serverconns, &cstruct);
+    afsconf_Close(tdir);
     if (code) {
 	ViceLog(0, ("vl_Initialize: ubik client init failed.\n"));
 	return code;
@@ -1653,16 +1667,22 @@ ReadSysIdFile()
 		 AFSDIR_SERVER_SYSID_FILEPATH, nentries));
 	return EIO;
     }
-    FS_HostAddr_cnt = nentries;
-    for (i = 0; i < nentries; i++) {
-	if (read(fd, (char *)&FS_HostAddrs[i], sizeof(afs_int32)) !=
-	    sizeof(afs_int32)) {
-	    ViceLog(0,
-		    ("%s: Read of addresses failed (%d)\n",
-		     AFSDIR_SERVER_SYSID_FILEPATH, errno));
-	    FS_HostAddr_cnt = 0;	/* reset it */
-	    return EIO;
+    if (FS_HostAddr_cnt == 0) {
+	FS_HostAddr_cnt = nentries;
+	for (i = 0; i < nentries; i++) {
+	    if (read(fd, (char *)&FS_HostAddrs[i], sizeof(afs_int32)) !=
+		sizeof(afs_int32)) {
+		ViceLog(0,
+			("%s: Read of addresses failed (%d)\n",
+			 AFSDIR_SERVER_SYSID_FILEPATH, errno));
+		FS_HostAddr_cnt = 0;	/* reset it */
+		return EIO;
+	    }
 	}
+    } else {
+	ViceLog(1,
+		("%s: address list ignored (NetInfo/NetRestrict override)\n",
+		 AFSDIR_SERVER_SYSID_FILEPATH));
     }
     close(fd);
     return 0;
@@ -1750,7 +1770,7 @@ Do_VLRegisterRPC()
 	FS_HostAddrs_HBO[i] = ntohl(FS_HostAddrs[i]);
     addrs.bulkaddrs_len = FS_HostAddr_cnt;
     addrs.bulkaddrs_val = (afs_uint32 *) FS_HostAddrs_HBO;
-    code = ubik_Call(VL_RegisterAddrs, cstruct, 0, &FS_HostUUID, 0, &addrs);
+    code = ubik_VL_RegisterAddrs(cstruct, 0, &FS_HostUUID, 0, &addrs);
     if (code) {
 	if (code == VL_MULTIPADDR) {
 	    ViceLog(0,
@@ -1775,6 +1795,43 @@ Do_VLRegisterRPC()
     return 0;
 }
 
+afs_int32
+SetupVL()
+{
+    afs_int32 code;
+    extern int rxi_numNetAddrs;
+    extern afs_uint32 rxi_NetAddrs[];
+
+#ifndef AFS_NT40_ENV
+    if (AFSDIR_SERVER_NETRESTRICT_FILEPATH || AFSDIR_SERVER_NETINFO_FILEPATH) {
+	/*
+	 * Find addresses we are supposed to register as per the netrestrict 
+	 * and netinfo files (/usr/afs/local/NetInfo and 
+	 * /usr/afs/local/NetRestict)
+	 */
+	char reason[1024];
+	afs_int32 code = parseNetFiles(FS_HostAddrs, NULL, NULL,
+				       ADDRSPERSITE, reason,
+				       AFSDIR_SERVER_NETINFO_FILEPATH,
+				       AFSDIR_SERVER_NETRESTRICT_FILEPATH);
+	if (code < 0) {
+	    ViceLog(0, ("Can't register any valid addresses: %s\n", reason));
+	    exit(1);
+	}
+	FS_HostAddr_cnt = (afs_uint32) code;
+    } else
+#endif
+    {
+	FS_HostAddr_cnt = rx_getAllAddr(FS_HostAddrs, ADDRSPERSITE);
+    }
+
+    if (FS_HostAddr_cnt == 1 && rxBind == 1)
+	code = FS_HostAddrs[0];
+    else 
+	code = htonl(INADDR_ANY);
+    return code;
+}
+
 afs_int32
 InitVL()
 {
@@ -1806,30 +1863,7 @@ InitVL()
     /* A good sysid file exists; inform the vlserver. If any conflicts,
      * we always use the latest interface available as the real truth.
      */
-#ifndef AFS_NT40_ENV
-    if (AFSDIR_SERVER_NETRESTRICT_FILEPATH || AFSDIR_SERVER_NETINFO_FILEPATH) {
-	/*
-	 * Find addresses we are supposed to register as per the netrestrict 
-	 * and netinfo files (/usr/afs/local/NetInfo and 
-	 * /usr/afs/local/NetRestict)
-	 */
-	char reason[1024];
-	afs_int32 code = parseNetFiles(FS_HostAddrs, NULL, NULL,
-				       ADDRSPERSITE, reason,
-				       AFSDIR_SERVER_NETINFO_FILEPATH,
-				       AFSDIR_SERVER_NETRESTRICT_FILEPATH);
-	if (code < 0) {
-	    ViceLog(0, ("Can't register any valid addresses: %s\n", reason));
-	    exit(1);
-	}
-	FS_HostAddr_cnt = (afs_uint32) code;
-    } else
-#endif
-    {
-	FS_HostAddr_cnt = rx_getAllAddr(FS_HostAddrs, ADDRSPERSITE);
-    }
 
-    FS_registered = 1;
     code = Do_VLRegisterRPC();
     return code;
 }
@@ -1854,6 +1888,7 @@ main(int argc, char *argv[])
 #endif
     int curLimit;
     time_t t;
+    afs_uint32 rx_bindhost;
 
 #ifdef	AFS_AIX32_ENV
     struct sigaction nsa;
@@ -2019,7 +2054,9 @@ main(int argc, char *argv[])
 #endif
     if (udpBufSize)
 	rx_SetUdpBufSize(udpBufSize);	/* set the UDP buffer size for receive */
-    if (rx_Init((int)htons(7000)) < 0) {
+    rx_bindhost = SetupVL();
+
+    if (rx_InitHost(rx_bindhost, (int)htons(7000)) < 0) {
 	ViceLog(0, ("Cannot initialize RX\n"));
 	exit(1);
     }
@@ -2036,11 +2073,12 @@ main(int argc, char *argv[])
     sc[1] = 0;			/* rxvab_NewServerSecurityObject(key1, 0) */
     sc[2] = rxkad_NewServerSecurityObject(rxkad_clear, NULL, get_key, NULL);
     sc[3] = rxkad_NewServerSecurityObject(rxkad_crypt, NULL, get_key, NULL);
-    tservice = rx_NewService( /* port */ 0, /* service id */ 1,	/*service name */
-			     "AFS",
-							/* security classes */ sc,
-							/* numb sec classes */
-			     4, RXAFS_ExecuteRequest);
+    tservice = rx_NewServiceHost(rx_bindhost,  /* port */ 0, /* service id */ 
+				 1,	/*service name */
+				 "AFS",
+				 /* security classes */ sc,
+				 /* numb sec classes */
+				 4, RXAFS_ExecuteRequest);
     if (!tservice) {
 	ViceLog(0,
 		("Failed to initialize RX, probably two servers running.\n"));
@@ -2096,7 +2134,18 @@ main(int argc, char *argv[])
     /* allow super users to manage RX statistics */
     rx_SetRxStatUserOk(fs_rxstat_userok);
 
+#if !defined(AFS_DEMAND_ATTACH_FS)
+    /* 
+     * For DAFS, we do not start the Rx server threads until after
+     * the volume package is initialized, and fileserver state is
+     * restored.  This is necessary in order to keep host and callback
+     * package state pristine until we have a chance to restore state.
+     *
+     * Furthermore, startup latency is much lower with dafs, so this
+     * shouldn't pose a serious problem.
+     */
     rx_StartServer(0);		/* now start handling requests */
+#endif
 
     /* we ensure that there is enough space in the vnode buffer to satisfy
      ** requests from all concurrent threads. 
@@ -2136,6 +2185,7 @@ main(int argc, char *argv[])
 	 * restore fileserver state */
 	fs_stateRestore();
     }
+    rx_StartServer(0);  /* now start handling requests */
 #endif /* AFS_DEMAND_ATTACH_FS */
 
     /*
@@ -2146,6 +2196,9 @@ main(int argc, char *argv[])
 
 #ifdef AFS_PTHREAD_ENV
     ViceLog(5, ("Starting pthreads\n"));
+#ifdef AFS_DEMAND_ATTACH_FS
+    FS_STATE_INIT;
+#endif
     assert(pthread_attr_init(&tattr) == 0);
     assert(pthread_attr_setdetachstate(&tattr, PTHREAD_CREATE_DETACHED) == 0);
 
