@@ -10,6 +10,8 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+RCSID
+    ("$Header: /cvs/openafs/src/vlserver/vlserver.c,v 1.22.2.3 2007/10/30 15:16:56 shadow Exp $");
 
 #include <afs/stds.h>
 #include <sys/types.h>
@@ -58,14 +60,12 @@ afs_uint32 HostAddress[MAXSERVERID + 1];
 extern int afsconf_CheckAuth();
 extern int afsconf_ServerAuth();
 
-static void *CheckSignal(void*);
+static CheckSignal();
 int LogLevel = 0;
 int smallMem = 0;
-int rxJumbograms = 0;		/* default is to not send and receive jumbo grams */
+int rxJumbograms = 1;		/* default is to send and receive jumbo grams */
 int rxMaxMTU = -1;
 afs_int32 rxBind = 0;
-int rxkadDisableDotCheck = 0;
-int debuglevel = 0;
 
 #define ADDRSPERSITE 16         /* Same global is in rx/rx_user.c */
 afs_uint32 SHostAddrs[ADDRSPERSITE];
@@ -76,15 +76,15 @@ CheckSignal_Signal()
     IOMGR_SoftSig(CheckSignal, 0);
 }
 
-static void *
-CheckSignal(void *unused)
+static
+CheckSignal()
 {
     register int i, errorcode;
     struct ubik_trans *trans;
 
     if (errorcode =
 	Init_VLdbase(&trans, LOCKREAD, VLGETSTATS - VL_LOWEST_OPCODE))
-	return (void *)errorcode;
+	return errorcode;
     VLog(0, ("Dump name hash table out\n"));
     for (i = 0; i < HASHSIZE; i++) {
 	HashNDump(trans, i);
@@ -93,7 +93,7 @@ CheckSignal(void *unused)
     for (i = 0; i < HASHSIZE; i++) {
 	HashIdDump(trans, i);
     }
-    return ((void *)ubik_EndTrans(trans));
+    return (ubik_EndTrans(trans));
 }				/*CheckSignal */
 
 
@@ -165,6 +165,7 @@ main(argc, argv)
     for (index = 1; index < argc; index++) {
 	if (strcmp(argv[index], "-noauth") == 0) {
 	    noAuth = 1;
+
 	} else if (strcmp(argv[index], "-p") == 0) {
 	    lwps = atoi(argv[++index]);
 	    if (lwps > MAXLWP) {
@@ -172,30 +173,22 @@ main(argc, argv)
 		       lwps, MAXLWP);
 		lwps = MAXLWP;
 	    }
-	} else if (strcmp(argv[index], "-d") == 0) {
-	    if ((index + 1) >= argc) {
-		fprintf(stderr, "missing argument for -d\n"); 
-		return -1; 
-	    }
-	    debuglevel = atoi(argv[++index]);
-	    LogLevel = debuglevel;
+
 	} else if (strcmp(argv[index], "-nojumbo") == 0) {
 	    rxJumbograms = 0;
-	} else if (strcmp(argv[index], "-jumbo") == 0) {
-	    rxJumbograms = 1;
+
 	} else if (strcmp(argv[index], "-rxbind") == 0) {
 	    rxBind = 1;
-	} else if (strcmp(argv[index], "-allow-dotted-principals") == 0) {
-	    rxkadDisableDotCheck = 1;
+
 	} else if (!strcmp(argv[index], "-rxmaxmtu")) {
 	    if ((index + 1) >= argc) {
 		fprintf(stderr, "missing argument for -rxmaxmtu\n"); 
 		return -1; 
 	    }
-	    rxMaxMTU = atoi(argv[++index]);
+	    rxMaxMTU = atoi(argv[++i]);
 	    if ((rxMaxMTU < RX_MIN_PACKET_SIZE) || 
 		(rxMaxMTU > RX_MAX_PACKET_DATA_SIZE)) {
-		printf("rxMaxMTU %d invalid; must be between %d-%d\n",
+		printf("rxMaxMTU %d% invalid; must be between %d-%d\n",
 		       rxMaxMTU, RX_MIN_PACKET_SIZE, 
 		       RX_MAX_PACKET_DATA_SIZE);
 		return -1;
@@ -209,9 +202,34 @@ main(argc, argv)
 	    strcpy(rxi_tracename, argv[++index]);
 
        } else if (strcmp(argv[index], "-auditlog") == 0) {
+	   int tempfd, flags;
+           FILE *auditout;
+           char oldName[MAXPATHLEN];
            char *fileName = argv[++index];
 
-           osi_audit_file(fileName);
+#ifndef AFS_NT40_ENV
+           struct stat statbuf;
+
+           if ((lstat(fileName, &statbuf) == 0) 
+               && (S_ISFIFO(statbuf.st_mode))) {
+               flags = O_WRONLY | O_NONBLOCK;
+           } else 
+#endif
+           {
+               strcpy(oldName, fileName);
+               strcat(oldName, ".old");
+               renamefile(fileName, oldName);
+               flags = O_WRONLY | O_TRUNC | O_CREAT;
+           }
+           tempfd = open(fileName, flags, 0666);
+           if (tempfd > -1) {
+               auditout = fdopen(tempfd, "a");
+               if (auditout) {
+                   osi_audit_file(auditout);
+               } else
+                   printf("Warning: auditlog %s not writable, ignored.\n", fileName);
+           } else
+               printf("Warning: auditlog %s not writable, ignored.\n", fileName);
 	} else if (strcmp(argv[index], "-enable_peer_stats") == 0) {
 	    rx_enablePeerRPCStats();
 	} else if (strcmp(argv[index], "-enable_process_stats") == 0) {
@@ -228,15 +246,15 @@ main(argc, argv)
 	    /* support help flag */
 #ifndef AFS_NT40_ENV
 	    printf("Usage: vlserver [-p <number of processes>] [-nojumbo] "
-		   "[-rxmaxmtu <bytes>] [-rxbind] [-allow-dotted-principals] "
-		   "[-auditlog <log path>] [-jumbo] [-d <debug level>] "
+		   "[-rxmaxmtu <bytes>] [-rxbind] "
+		   "[-auditlog <log path>] "
 		   "[-syslog[=FACILITY]] "
 		   "[-enable_peer_stats] [-enable_process_stats] "
 		   "[-help]\n");
 #else
 	    printf("Usage: vlserver [-p <number of processes>] [-nojumbo] "
-		   "[-rxmaxmtu <bytes>] [-rxbind] [-allow-dotted-principals] "
-		   "[-auditlog <log path>] [-jumbo] [-d <debug level>] "
+		   "[-rxmaxmtu <bytes>] [-rxbind] "
+		   "[-auditlog <log path>] "
 		   "[-enable_peer_stats] [-enable_process_stats] "
 		   "[-help]\n");
 #endif
@@ -335,7 +353,7 @@ main(argc, argv)
     ubik_CheckRXSecurityProc = afsconf_CheckAuth;
     ubik_CheckRXSecurityRock = (char *)tdir;
     code =
-	ubik_ServerInitByInfo(myHost, htons(AFSCONF_VLDBPORT), &info, clones,
+	ubik_ServerInitByInfo(myHost, htons(AFSCONF_VLDBPORT), &info, &clones,
 			      vl_dbaseName, &VL_dbase);
     if (code) {
 	printf("vlserver: Ubik init failed with code %d\n", code);
@@ -355,10 +373,7 @@ main(argc, argv)
     sc[0] = rxnull_NewServerSecurityObject();
     sc[1] = (struct rx_securityClass *)0;
     sc[2] = rxkad_NewServerSecurityObject(0, tdir, afsconf_GetKey, NULL);
-#ifdef USE_RXKAD_KEYTAB
-    if (rxkad_InitKeytabDecrypt(AFSDIR_SERVER_RXKAD_KEYTAB_FILEPATH) == 0)
-	rxkad_BindKeytabDecrypt(sc[2]);
-#endif
+
     tservice =
 	rx_NewServiceHost(host, 0, USER_SERVICE_ID, "Vldb server", sc, 3,
 		      VL_ExecuteRequest);
@@ -371,12 +386,6 @@ main(argc, argv)
 	lwps = 4;
     rx_SetMaxProcs(tservice, lwps);
 
-    if (rxkadDisableDotCheck) {
-        rx_SetSecurityConfiguration(tservice, RXS_CONFIG_FLAGS,
-                                    (void *)RXS_CONFIG_FLAGS_DISABLE_DOTCHECK,
-                                    NULL);
-    }
-
     tservice =
 	rx_NewServiceHost(host, 0, RX_STATS_SERVICE_ID, "rpcstats", sc, 3,
 		      RXSTATS_ExecuteRequest);
@@ -400,6 +409,4 @@ main(argc, argv)
     rx_SetRxStatUserOk(vldb_rxstat_userok);
 
     rx_StartServer(1);		/* Why waste this idle process?? */
-
-    return 0; /* not reachable */
 }
